/**************************************************************************************

        PROTUX - THE FREE PROFESSIONAL AUDIO TOOLS FOR LINUX
        AUTHOR : See AUTHORS file for details

        This software is distributed under the terms of the GNU General Public License
        as specified in the COPYING file.

***************************************************************************************/
#include <cmath>
#include <mustux.h>
#include "AudioClip.hh"
#include "ColorManager.hh"

AudioClip::AudioClip(Track* pParentTrack,
                     Audio *pAudioSource,
                     long long pSourceFirstBlock,
                     long long pLenght,
                     long long pTrackInsertBlock)
        : MustuxDrawableRegion( pParentTrack->get_drawarea() ) ,
          MustuxObject()
        {
        PENTERCONS;
        parentTrack = pParentTrack;
        parentSong  = parentTrack->parentSong;
        audioSource = pAudioSource;
        clipName = audioSource->file->filename;
        blockSize = audioSource->file->blockSize;
        channels = audioSource->file->channels;
        rate = audioSource->file->rate;
        bitDepth = audioSource->file->bitDepth;
        isTake=false;
        maxSample = (int) ( pow(2,bitDepth) / 256 );
        clipGainFactor = 1.0;

        sourceFirstBlock= pSourceFirstBlock;
        long long lenght = pLenght;
        sourceLastBlock = sourceFirstBlock + lenght;
        trackFirstBlock = pTrackInsertBlock;
        trackLastBlock = trackFirstBlock + lenght;
        isMuted=false;
        set_blur(false);
        PEXITCONS;
        }


AudioClip::~AudioClip()
        {
        PENTERDES;
        PEXITDES;
        }


Audio* AudioClip::get_audio_source()
        {
        return audioSource;
        }




void AudioClip::update()
        {
        int baseX = parentSong->block_to_xpos(trackFirstBlock);
        int zoomStep = parentSong->get_zoom_step();
        int clipXWidth = ( sourceLastBlock - sourceFirstBlock )  / zoomStep;
        drawArea->update(baseX, parentTrack->real_baseY(), clipXWidth, parentTrack->height);
        }



void AudioClip::clear()
        {
        int baseX = parentSong->block_to_xpos(trackFirstBlock);
        int zoomStep = parentSong->get_zoom_step();
        int clipXWidth = ( sourceLastBlock - sourceFirstBlock )  / zoomStep;
        drawArea->clear(baseX, parentTrack->real_baseY(), clipXWidth, parentTrack->height);
	drawArea->painter->flush();
        }


int AudioClip::draw()
        {
        QPainter* painter = drawArea->painter;
        int hzoom = parentSong->get_hzoom();
        int zoomStep = parentSong->get_zoom_step();
        int baseY  = this->get_baseY();
        int height = this->get_height();
        int baseX =  this->get_baseX();
        int clipXWidth = this->get_width();
        int trackWidth = drawArea->width();

        long long firstBlock = sourceFirstBlock;

        if ( baseX < 0)
                {
                firstBlock += (-1*baseX) * zoomStep;
                clipXWidth -= (-1*baseX);
                baseX = 0;
                }

        if (clipXWidth > (trackWidth - baseX))
                clipXWidth = trackWidth - baseX;

        if (clipXWidth<0)
                return 1;

        // clip info area bg
        if (parentTrack->isActive)
                painter->fillRect( baseX , baseY, clipXWidth, 16, QColor(255,0,0));
        else
                painter->fillRect( baseX , baseY, clipXWidth, 16, ColorManager::get(ColorManager::COLOR_CLIP_INFO_AREA_BG));



        // Draw Clip Info Area
        if (clipXWidth>70)
                {
                painter->setPen(ColorManager::get(ColorManager::COLOR_CLIP_TEXT_FG));
                drawArea->painter->setFont( QFont( "Helvetica", 7 ) );
                for (int i=0; i<channels; i++)
                        painter->drawEllipse( baseX + 5 + 4*i, baseY + 5, 8 , 8);
                if (isMuted)
                        painter->drawText( baseX + 23, baseY + 12,"M");
                QString sRate; sRate.setNum(rate);             painter->drawText( baseX + 40, baseY + 12, sRate);
                QString sBitDepth; sBitDepth.setNum(bitDepth); painter->drawText( baseX + 80, baseY + 12, sBitDepth);
                QString sourceType=(isTake?"CAP":"SRC");       painter->drawText( baseX + 105, baseY + 12, sourceType);

                // Clip Gain
                QString sclipGain; sclipGain.setNum((double)clipGainFactor,'f',1); sclipGain = "G:"+sclipGain;
                painter->drawText (baseX + 135, baseY + 12, sclipGain);

                // Clip Name
                painter->drawText( baseX + 175, baseY + 12, clipName);
                }

        //Draw axis
        //painter->setPen(QColor(30,70,50));
        //painter->drawLine(baseX, baseY + yc, clipXWidth , baseY + yc);

        // Check Cross Fades
        AudioClip* pClip = this->prev;
        AudioClip* nClip = this->next;
        long long pTLB = pClip ? pClip->trackLastBlock : 0;
        long long nTFB = nClip ? nClip->trackFirstBlock : 0;
        bool crossAtLeft  = ((pClip) && (pTLB > trackFirstBlock));
        bool crossAtRight = ((nClip) && (nTFB < trackLastBlock));
        int lCrossX = crossAtLeft  ? parentSong->block_to_xpos(pTLB) - parentSong->block_to_xpos(trackFirstBlock) : 0;
        int rCrossX = crossAtRight ? parentSong->block_to_xpos(trackLastBlock) - parentSong->block_to_xpos(nTFB)  : 0;

        // main bg (dont paint under crossfades)
        if (isMuted)
                painter->fillRect( baseX + lCrossX, baseY+16, clipXWidth - rCrossX - lCrossX, height-16, ColorManager::get(ColorManager::COLOR_CLIP_MUTED_BG));
        else if (isSelected)
                painter->fillRect( baseX + lCrossX, baseY+16, clipXWidth - rCrossX - lCrossX, height-16, ColorManager::get(ColorManager::COLOR_CLIP_SELECTED_BG));
        else
                painter->fillRect( baseX + lCrossX, baseY+16, clipXWidth - rCrossX - lCrossX, height-16 , ColorManager::get(ColorManager::COLOR_CLIP_BG));

        // bg under RIGHT crossfade region  ( only RIGHT !)
        painter->fillRect( baseX + clipXWidth - rCrossX , baseY+16, rCrossX, height-16, QColor(120,150,130));


        // Draw the peaks
        int peaksToFill = clipXWidth;
        char* peakBuffer = new char[ peaksToFill * channels ];
        Peak* pk = audioSource->peak;
        if (!pk) // There is no peak for this audio Clip. Maybe this is just a recording box... No
                 // need to worry ...
                return -1;

        int peaksFilled = pk->fill_buffer(peakBuffer, firstBlock, peaksToFill, hzoom);
        const int chHeight = (height-16) / channels;
        //const int ycenter = chHeight / 2 + 16;
        int dy = 0;
        int yc = 0;
        long long ppos=0;
        char sample;

	if ( hzoom <= Peak::MAX_ZOOM_USING_SOURCEFILE ) // MICRO VIEW MODE
                {
                const float yFactor = (float) chHeight / maxSample;
                // Set peaks foreground color
                int* lastdy = new int[channels];
                for (int k=0; k<channels; k++)
                        lastdy[k]=0;

                // peaks outside cross fade regions
                painter->setPen(ColorManager::get(ColorManager::COLOR_CLIP_PEAK_MICROVIEW_FG));
                for (int x=baseX; x<baseX + peaksFilled - rCrossX; x++)
                        {
                        for (int k=0; k<channels; k++)
                                {
                                yc = chHeight/2 + chHeight*k + 16;
                                sample = (char) ( peakBuffer[ppos] * clipGainFactor * -1); //either micro or macroview is painted upsidedown. macroview has only positive peaks, so we invert microview??
                                dy = (int) (yFactor * (float) sample ) ;
                                painter->drawLine( x-1 , baseY + yc + lastdy[k] , x , baseY + yc + dy );
                                lastdy[k]=dy;
                                ppos++;
                                }
                        }
                // peaks under right cross fade region ( only right needed )
                if (crossAtRight)
                        {
                        painter->setPen(Qt::black);
                        for (int x=baseX + peaksFilled - rCrossX; x < baseX + peaksFilled; x++)
                                {
                                for (int k=0; k<channels; k++)
                                        {
                                        yc = chHeight/2 + chHeight*k + 16;
                                        sample = (char) ( peakBuffer[ppos] * clipGainFactor * -1); //either micro or macroview is painted upsidedown. macroview has only positive peaks, so we convert microview??
                                        dy = (int) (yFactor * (float) sample ) ;
                                        painter->drawLine( x-1 , baseY + yc + lastdy[k] , x , baseY + yc + dy );
                                        lastdy[k]=dy;
                                        ppos++;
                                        }
                                }
                        }
                delete lastdy;
                }
        else
                {
                // peaks outside cross fade regions
                const float yFactor = (float) 2.0 * chHeight / maxSample;
                painter->setPen(ColorManager::get(ColorManager::COLOR_CLIP_PEAK_MACROVIEW_FG));
                for (int x=baseX; x<baseX + peaksFilled - rCrossX; x++)
                        {
                        for (int k=0; k<channels; k++)
                                {
                                yc = chHeight*(k+1)+ 16;
                                sample = peakBuffer[ppos];
                                if (sample>=127)
                                        painter->setPen(ColorManager::get(ColorManager::COLOR_CLIP_PEAK_OVERLOADED_SAMPLE));
                                else
                                        painter->setPen(ColorManager::get(ColorManager::COLOR_CLIP_PEAK_MACROVIEW_FG));
                                dy = (int) (yFactor * (float) sample ) ;
                                painter->drawLine( x , baseY + yc, x , baseY + yc - dy );
                                ppos++;
                                }
                        }
                // peaks under right cross fade region ( only right needed )
                if (crossAtRight)
                        {
                        painter->setPen(Qt::black);
                        for (int x=baseX + peaksFilled - rCrossX; x < baseX + peaksFilled; x++)
                                {

                                for (int k=0; k<channels; k++)
                                        {
                                        yc = chHeight*(k+1)+ 16;
                                        sample = peakBuffer[ppos];
                                        dy = (int) (yFactor * (float) sample ) ;
                                        painter->drawLine( x , baseY + yc, x , baseY + yc - dy );
                                        ppos++;
                                        }
                                }
                        }
                }
        delete peakBuffer;

        // Black Contour of clip Info area
        painter->setPen(ColorManager::get(ColorManager::COLOR_CLIP_TEXT_FG)); // CHANGE TO CLIP_COUNTOUR
        painter->drawRect(baseX, baseY , clipXWidth , 16);

        if (crossAtLeft)
                        {
                        painter->setPen(Qt::blue);
                        painter->drawArc(baseX - lCrossX, baseY + 16, lCrossX * 2 , (height-16)*2 , 90*16, -90*16);
                        }
        if (crossAtRight)
                        {
                        painter->setPen(Qt::white);
                        painter->drawArc(baseX + clipXWidth - rCrossX, baseY + 16, rCrossX * 2, (height-16) * 2, 90*16, 90*16);
                        }

        // Black Contour of clip
        painter->setPen(ColorManager::get(ColorManager::COLOR_CLIP_TEXT_FG)); // CHANGE TO CLIP_COUNTOUR
        painter->drawRect(baseX, baseY , clipXWidth , height);

        return 1;
        }


void AudioClip::set_track_first_block(long long newTrackFirstBlock)
        {
        long long lenght = sourceLastBlock - sourceFirstBlock;
        trackFirstBlock=newTrackFirstBlock;
        trackLastBlock=trackFirstBlock + lenght;
        }


int AudioClip::play()
        {
        return 1;
        }

int AudioClip::split()
        {
        return 1;
        }

void AudioClip::set_muted(bool b)
        {
        isMuted=b;
        }





void AudioClip::set_left_edge(long long block)
        {
        long long olen = sourceLastBlock - sourceFirstBlock;
        long long otfb = trackFirstBlock;
        trackFirstBlock = block;
        long long lenght = olen + otfb - block;
        sourceFirstBlock = sourceLastBlock - lenght;
        }


void AudioClip::set_right_edge(long long block)
        {
        long long lenght = sourceLastBlock - sourceFirstBlock;
        long long olen = lenght;
        long long otlb = trackLastBlock;
        trackLastBlock = block;
        lenght = olen + block - otlb;
        sourceLastBlock = sourceFirstBlock + lenght;
        }


void AudioClip::set_first_source_block(long long block)
        {
        // TODO
        }


void AudioClip::set_last_source_block(long long block)
        {
        sourceLastBlock = block;
        long long lenght = sourceLastBlock - sourceFirstBlock;
        trackLastBlock = trackFirstBlock + lenght;
        }


long long AudioClip::get_lenght()
        {
        return sourceLastBlock - sourceFirstBlock;
        }
/*
void AudioClip::extend_to_end() // DEPRECATED
        {
        long long block = audioSource->get_last_block();
        set_last_source_block(block);
        }
*/

void AudioClip::set_blur(bool stat)
        {
        // blur off
        ColorManager::set_blur(ColorManager::COLOR_CLIP_RECORDING             , stat );
        ColorManager::set_blur(ColorManager::COLOR_CLIP_BG                    , stat );
        ColorManager::set_blur(ColorManager::COLOR_CLIP_SELECTED_BG           , stat );
        ColorManager::set_blur(ColorManager::COLOR_CLIP_MUTED_BG              , stat );
        ColorManager::set_blur(ColorManager::COLOR_CLIP_INFO_AREA_BG          , stat );
        ColorManager::set_blur(ColorManager::COLOR_CLIP_PEAK_MICROVIEW_FG     , stat );
        ColorManager::set_blur(ColorManager::COLOR_CLIP_PEAK_MACROVIEW_FG     , stat );
        ColorManager::set_blur(ColorManager::COLOR_CLIP_TEXT_FG               , stat );
        ColorManager::set_blur(ColorManager::COLOR_CLIP_TEXT_FG               , stat );
        ColorManager::set_blur(ColorManager::COLOR_CLIP_PEAK_OVERLOADED_SAMPLE, stat );
        }


int AudioClip::get_baseX()
        {
        return parentSong->block_to_xpos(trackFirstBlock);
        }

int AudioClip::get_baseY()
        {
        return parentTrack->real_baseY() + 3;
        }

int AudioClip::get_width()
        {
        return ( sourceLastBlock - sourceFirstBlock ) / parentSong->get_zoom_step();
        }


int AudioClip::get_height()
        {
        return parentTrack->height - 8;
        }


// eof


