/**************************************************************************************

        PROTUX - THE FREE PROFESSIONAL AUDIO TOOLS FOR LINUX
        AUTHOR : See AUTHORS file for details

        This software is distributed under the terms of the GNU General Public License
        as specified in the COPYING file.

***************************************************************************************/
#include <qcolor.h>
#include <mustux.h>
#include "BusMonitor.hh"
#include "Project.hh"

#include <stdlib.h>

#define MAX_LEVEL 50
#define BASEX_IN 10
#define BASEY_IN 35
#define BASEX_OUT 140
#define BASEY_OUT 35
#define HEIGHT 33
#define WIDTH 120
#define SPACING 2
#define TEXT_COLOR Qt::black
#define LEVEL_CLEAR_COLOR QColor(60,70,60)

// This is for VU : db = 20 * log ( sample / MaxSample )

        // there is a wrong concept in this class. Bus Monitor is monitoring
        // the mixer !! (and not the buses... :-)
        // change it ASAP.

BusMonitor::BusMonitor(MustuxDrawable* pDrawable, Interface* pAssocInterface)
        {
        PENTERCONS;
        drawable = pDrawable;
        assocInterface = pAssocInterface;
        for (int j=0; j < MustuxAudioDeviceMapper::MAX_BUSES; j++)
                for (int i=0; i < MAX_CHANNELS_PER_BUS; i++)
                        {
                        levelOut[j][i] = 0;
                        levelIn[j][i] = 0;
                        }

        updateTimer = new QTimer(this);
        connect( updateTimer , SIGNAL(timeout()), this, SLOT(update()));
        // FOR LATER spectrogram=0;
        PEXITCONS;
        }


BusMonitor::~BusMonitor()
        {
        PENTERDES;
        updateTimer->stop();
        delete updateTimer;
        PEXITDES;
        }


void BusMonitor::init()
        {
        updateTimer->start(50);
        refresh();
        }


void BusMonitor::refresh()
        {
        PENTER3;
        // draw (and clear) output buses
        int pos=0;
        for (int b=0; b<MustuxAudioDeviceMapper::get_total_buses_out(); b++)
                {
                if (!assocInterface->get_current_project()->get_current_song()->get_mixer()->valid_playback_bus(b))
                        continue;
                bool ab = MustuxAudioDeviceMapper::is_bus_allocated_for_playback(b);
                draw_bus_vu_base(BASEX_OUT,
                                 BASEY_OUT,
                                 MustuxAudioDeviceMapper::is_stereo_bus_out(b) ? 2 : 1,
                                 pos,
                                 MustuxAudioDeviceMapper::get_bus_out_name(b),
                                 ab);
                pos++;
                }

        // draw (and clear) input buses
        pos=0;
        for (int b=0; b<MustuxAudioDeviceMapper::get_total_buses_in(); b++)
                {
                if (!assocInterface->get_current_project()->get_current_song()->get_mixer()->valid_capture_bus(b))
                        continue;
                bool ab = MustuxAudioDeviceMapper::is_bus_allocated_for_capture(b);
                draw_bus_vu_base(BASEX_IN,
                                 BASEY_IN,
                                 MustuxAudioDeviceMapper::is_stereo_bus_in(b) ? 2 : 1,
                                 pos,
                                 MustuxAudioDeviceMapper::get_bus_in_name(b),
                                 ab);
                pos++;
                }
        PEXIT3;
        }



void BusMonitor::update()
        {
        PENTER4;
        if (!MustuxAudioDeviceMapper::is_any_bus_allocated())
                {
                PEXIT4;
                return;
                }

        QPainter* p = drawable->fastPainter;

        // update out levels
        for (int busOut=0; busOut<MustuxAudioDeviceMapper::get_total_buses_out(); busOut++)
                {
                if (!MustuxAudioDeviceMapper::is_bus_allocated_for_playback(busOut))
                        continue;

                char* buf = MustuxAudioDeviceMapper::get_bus_out_transfer_buffer(busOut, MustuxAudioDeviceMapper::STEREO); // IMPROVE-ME...
                int bufSize = MustuxAudioDeviceMapper::get_bus_out_transfer_size(busOut);
                int channels = MustuxAudioDeviceMapper::is_stereo_bus_out(busOut) ? 2 : 1;

                char* p  = buf;
                int* max = new int[channels];
                for (int k=0; k<channels; k++) max[k]=0;
                short sho;
                int totalsamples = bufSize/(channels*sizeof(short));
                for (int i=0; i <totalsamples; i++)
                        {
                        for (int k=0; k<channels; k++)
                                {
                                sho = (short) *(short*)p;
                                if (abs(sho) > max[k]) max[k] = (int)abs(sho);
                                p+=sizeof(short);
                                }
                        }
                for (int k=0; k<channels; k++)
                        {
                        float f = (float) max[k] / MAX_SAMPLE;
                        levelOut[busOut][k] = (int) (f*MAX_LEVEL);
                        }
                delete max;
                }

        // update in levels
        for (int busIn=0; busIn<MustuxAudioDeviceMapper::get_total_buses_in(); busIn++)
                {
                if (!MustuxAudioDeviceMapper::is_bus_allocated_for_capture(busIn))
                        continue;

                char* buf = MustuxAudioDeviceMapper::get_bus_in_transfer_buffer(busIn, MustuxAudioDeviceMapper::STEREO); // IMPROVE-ME...
                int bufSize = MustuxAudioDeviceMapper::get_bus_in_transfer_size(busIn);
                int channels = MustuxAudioDeviceMapper::is_stereo_bus_in(busIn) ? 2 : 1;

                char* p  = buf;
                int* max = new int[channels];
                for (int k=0; k<channels; k++) max[k]=0;

                short sho;

                int totalsamples = bufSize/(channels*sizeof(short));
                for (int i=0; i <totalsamples; i++)
                        {
                        for (int k=0; k<channels; k++)
                                {
                                sho = (short) *(short*)p;
                                if (abs(sho) > max[k]) max[k] = (int)abs(sho);
                                p+=sizeof(short);
                                }
                        }
/*
                // Show buffer snapshot (PUT ME ON MUSTUXDEBUGGER)
                for (int o=0; o<bufSize; o++)
                        if (o%400==0)
                                printf("%d",buf[o]);
                printf("\n");

                for (int k=0; k<channels; k++)
                        printf("MAX[%d]=%d    ",k,max[k]);
                printf("\n\n");
*/

                for (int k=0; k<channels; k++)
                        {
                        float f = (float) max[k] / MAX_SAMPLE;
                        f = f>1.0?1.0:f;
                        levelIn[busIn][k] = (int) (f*MAX_LEVEL);
                        }
                delete max;
                }


        // draw playback buses levels
        int pos=0;
        for (int busOut = 0; busOut < MustuxAudioDeviceMapper::get_total_buses_out(); busOut++)
                {
                if (!assocInterface->get_current_project()->get_current_song()->get_mixer()->valid_playback_bus(busOut))
                        continue;
                if (MustuxAudioDeviceMapper::is_bus_allocated_for_playback(busOut))
                        {
                        int channels = MustuxAudioDeviceMapper::is_stereo_bus_out(busOut) ? 2 : 1;
                        draw_bus_vu_peaks(MustuxAudioDeviceMapper::PLAYBACK,
                                  levelOut[busOut],
                                  BASEX_OUT,
                                  BASEY_OUT,
                                  channels,
                                  pos);
                        }
                pos++;
                }

        // draw capture buses levels
        pos = 0;
        for (int busIn = 0; busIn < MustuxAudioDeviceMapper::get_total_buses_in(); busIn++)
                {
                if (!assocInterface->get_current_project()->get_current_song()->get_mixer()->valid_capture_bus(busIn))
                        continue;
                if (MustuxAudioDeviceMapper::is_bus_allocated_for_capture(busIn))
                        {

                        int channels = MustuxAudioDeviceMapper::is_stereo_bus_in(busIn) ? 2 : 1;
                        draw_bus_vu_peaks(MustuxAudioDeviceMapper::CAPTURE,
                                  levelIn[busIn],
                                  BASEX_IN,
                                  BASEY_IN,
                                  channels,
                                  pos);
                        }
                pos++;
                }
        PEXIT4;
        }



void BusMonitor::draw_bus_vu_base(int bx, int by, int channels, int pos , QString title, bool isActiveBus)
        {
        PENTER3;
        QPainter* p = drawable->painter;
        p->setFont(QFont("Fixed",6));
        int y = by + pos * (HEIGHT + SPACING);
        p->setPen(QColor(20,20,45));
        QColor c(140+(isActiveBus?100:0),160,120);
        p->fillRect( bx , y - HEIGHT , WIDTH , HEIGHT , c );
        p->setPen(QColor(200,255,200));
        p->drawLine( bx, y - HEIGHT, bx, y );
        p->drawLine( bx, y - HEIGHT, bx + WIDTH , y - HEIGHT );
        p->setPen(QColor(20,80,20));
        p->drawLine( bx + WIDTH , y - HEIGHT , bx + WIDTH , y);
        p->drawLine( bx , y , bx + WIDTH , y );
        p->setPen(TEXT_COLOR);
        p->drawText( bx + 4, y - 26, WIDTH, 6, Qt::AlignLeft | Qt::DontClip, title);
        p->setPen(LEVEL_CLEAR_COLOR);
        for (int i=0; i<MAX_LEVEL; i++)
                {
                for (int k=0; k<channels; k++)
                        {
                        int x = bx + i * 2 + 4;
                        p->drawLine(x , y + k*8 - 13,
                                    x , y + k*8 - 17);
                        }
                }
        drawable->update();
        PEXIT3;
        }

// eof



void BusMonitor::draw_bus_vu_peaks(int stream, int* levelBuffer, int bx, int by, int channels, int pos)
        {
        PENTER3;
        QPainter* p = drawable->fastPainter;
        int y = by + pos * (HEIGHT + SPACING);
        p->setFont( QFont( "Helvetica", 6) );

        // First clear where is not used
        p->setPen(LEVEL_CLEAR_COLOR);
        for (int k=0; k<channels; k++)
                {
                for (int i=0 /*WAS levelBuffer[k]*/; i<MAX_LEVEL; i++)
                        {
                        int x = bx + i * 2 + 4;
                        p->drawLine(x , y + k*8 - 13,
                                    x , y + k*8 - 17);
                        }
                }
        // check overload
        for (int k=0; k<channels; k++)
                {
                if (levelBuffer[k]>=MAX_LEVEL)
                        p->fillRect( bx + MAX_LEVEL * 2 + 5, y + k*8 - 17, 8 , 5, Qt::red);
                }
        // draw levels
        for (int k=0; k<channels; k++)
                {
                for (int i = 0; i < levelBuffer[k]; i++)
                        {
                        int cR = 0;// WAS i > (MAX_LEVEL / 2) ? (510 / MAX_LEVEL) * i - 255 : 0;
                        int cG = 255; // WAS 255 - (i > (MAX_LEVEL / 2) ? (510 / MAX_LEVEL) * i - 255 : 0);
                        //WAS if (cR > 255) cR=255;
                        //WAS if (cG > 255) cG=255;
                        p->setPen(QColor(cR, cG, 0));
                        int x = bx + i * 2 + 4;
                        p->drawLine(x , y + k*8 - 13, x , y + k*8 - 17);
                        }
                }
        PEXIT3;
        }


//eof

