/**************************************************************************************

        PROTUX - THE FREE PROFESSIONAL AUDIO TOOLS FOR LINUX
        AUTHOR : See AUTHORS file for details

        This software is distributed under the terms of the GNU General Public License
        as specified in the COPYING file.

***************************************************************************************/

#include <sys/vfs.h>
#include "Interface.hh"
#include "ProjectManager.hh"
#include "ColorManager.hh"


Interface::Interface(QWidget *parent) : MustuxInterface( parent )
        {
        PENTERCONS;

        resize(MINIMUM_INTERFACE_WIDTH,MINIMUM_INTERFACE_HEIGHT);
        setMinimumSize(MINIMUM_INTERFACE_WIDTH,MINIMUM_INTERFACE_HEIGHT);
        setIcon(QPixmap (RESOURCES_DIR "/protux.xpm") );

        ColorManager::init();

        // top level container
        mainVBoxLayout = new QVBoxLayout(this, 6);
        mainVBoxLayout->setResizeMode(QLayout::FreeResize);

        QWidget* hbox = new QWidget( this );
        hbox->setFixedHeight(105);  //TODO : Must become a Global Properties property
        QHBoxLayout* hboxlayout = new QHBoxLayout( hbox );

        infoLcd = new MustuxLcd(500, 100, hbox , "infoLcd");  //TODO : Must become a Global Properties property
        //infoLcd->setMaximumWidth(500);
        hboxlayout->addWidget(infoLcd);
        set_default_lcd(infoLcd);

        dataBox = new MustuxDrawable(120,100,hbox,"dataBox");
        dataBox->set_background_color(150,170,150);
        hboxlayout->addWidget(dataBox);

        busBox = new MustuxDrawable(120,100,hbox,"busBox");
        hboxlayout->addWidget(busBox);

        spectrogram = (Spectrogram*) 0;
        /* FOR LATER
        spectrogram = new Spectrogram( 100,100, hbox, "Spectrogram");
        spectrogram ->setFixedWidth(GlobalProperties::get_int("SPECTROGRAM_WIDTH"));//TOGP
        hboxlayout->addWidget(spectrogram);
        */
        mainVBoxLayout->addWidget( hbox );

        busMonitor = new BusMonitor(busBox,this);

        projectManager = new ProjectManager(this);

        // Try to load the "Untitled" project from a previous session. If it cant, it creates a new one with 1 song
	if (projectManager->projectExists("Untitled"))
	        projectManager->load_project("Untitled");
	else
	        projectManager->create_new_project("Untitled", 1);

        QWidget* hbox2 = new QWidget(this);
        hbox2->setFixedHeight(20);
        QHBoxLayout* hbox2layout = new QHBoxLayout( hbox2 );

/*        QLabel* l1 = new QLabel("PILL",hbox2);
        QToolTip::add( l1,  "This will be the Protux InternaL Language console (Accepting suggestions to change the language name :-)");
        hbox2layout->addWidget( l1 );
        QLineEdit* prompt = new QLineEdit(hbox2);
        hbox2layout->addWidget( prompt );
*/
        ramUsage = new QLabel("",hbox2);
        QToolTip::add( ramUsage,  "How much RAM protux is using right now");
        ramUsage->setFrameStyle( QFrame::Panel | QFrame::Sunken );
        ramUsage->setLineWidth( 1 );
        ramUsage->setFixedWidth(100);
        hbox2layout->addWidget( ramUsage );

        snapStatus = new QLabel("",hbox2);
        QToolTip::add( snapStatus,  "Snap status : >|< Snap to work cursor  >[]< Snap to clips borders");
        snapStatus->setFrameStyle( QFrame::Panel | QFrame::Sunken );
        snapStatus->setLineWidth( 1 );
        snapStatus->setFixedWidth(90);
        hbox2layout->addWidget( snapStatus );

        freeTime = new QLabel("",hbox2);
        QToolTip::add( freeTime,  "How much record time you still have");
        freeTime->setFrameStyle( QFrame::Panel | QFrame::Sunken );
        freeTime->setLineWidth( 1 );
        freeTime->setFixedWidth(100);
        hbox2layout->addWidget( freeTime );

        diskUsage = new QLabel("",hbox2);
        QToolTip::add( diskUsage,  "How many disk space (Mb) protuxproject dir is using");
        diskUsage->setFrameStyle( QFrame::Panel | QFrame::Sunken );
        diskUsage->setLineWidth( 1 );
        diskUsage->setFixedWidth(100);
        hbox2layout->addWidget( diskUsage );

        diskSpace = new QLabel("",hbox2);
        QToolTip::add( diskSpace,  "How much free space there is on the same HD where protuxprojects dir is");
        diskSpace->setFrameStyle( QFrame::Panel | QFrame::Sunken );
        diskSpace->setLineWidth( 1 );
        diskSpace->setFixedWidth(100);
        hbox2layout->addWidget( diskSpace );

        mainVBoxLayout->addWidget( hbox2 );


        helpWindow = new Help(RESOURCES_DIR "/help.text", (QWidget*) 0);

        QString msg = "Welcome to Protux ";
        msg.append(VERSION);
        msg.append("       Press H for Quick Help                 ");
        infoLcd->roll_message((const char*) msg.latin1());

        jmb->init(RESOURCES_DIR "/jmb.map");

        isHelpHidden=true;

        globalPropertiesDialog = new GlobalPropertiesDialog(this);

        tuner = (MustuxTuner*) 0;

        showMaximized();

        busMonitor->init();

        statusBarTimer = new QTimer(this);
        connect( statusBarTimer , SIGNAL(timeout()), this, SLOT(update_system_resources()));
        statusBarTimer->start(1000);

        show_splash_screen();
        PEXITCONS;
        }


Interface::~Interface()
        {
        PENTERDES;
        delete globalPropertiesDialog;
        delete busMonitor;
        delete helpWindow;
        delete projectManager;
        PEXITDES;
        }


int Interface::process_jmb_action(MustuxJogMouseBoardMessage* jmbMesg)
        {
        int r=-1;
        int action = jmbMesg->action;
        bool isBeginning = jmbMesg->moment;
        QString actionName = jmbMesg->label;
        int parameter = jmbMesg->parameter;
        int collNumber = jmbMesg->collectedNumber;

        // Interface scope actions
        if (actionName == "QUIT")
                {
                if (currentProject)
                        currentProject->save();
                QApplication::exit();
                }
        else if (actionName == "ABOUT")
                {
                about_protux();
                }
        else if (actionName == "FILE STUFF")
                {
                projectManager->show();
		projectManager->raise();
                r=1;
                }
        else if (actionName == "LOCK/UNLOCK EVERYTHING")
                {
                if (jmb->is_locked())
                        jmb->unlock();
                else
                        jmb->lock();
                r=1;
                }

        else if (actionName == "HELP")
                {
                help();
                r=1;
                }

        else if (actionName == "TUNER")
                {
                if (tuner)
                        {
                        delete tuner;
                        tuner = (MustuxTuner*) 0;
                        }
                else
                        {
                        tuner = new MustuxTuner(1,true);
                        tuner->init_frontend();
                        tuner->show();
                        }

                r=1;
                }

        else if (actionName == "GLOBAL PROPERTIES")
                {
                globalPropertiesDialog->show();
                r=1;
                }
        else
                {
                if (currentProject)
                        {
                        // PROJECT SCOPE ACTIONS
                        if (actionName == "SAVE CURRENT PROJECT")
                                {
                                r = currentProject->save();
                                }
                        else if (actionName == "RENDER SONG")
                                {
                                r = currentProject->get_current_song()->render();
                                }
                        else
                                {
                                // SONG SCOPE ACTIONS
                                Song* currentSong = currentProject->get_current_song();
                                r = currentSong->process_jmb_action(jmbMesg);
                                }
                        }
                else
                        {
                        info("OPEN OR CREATE A PROJECT FIRST!!");
                        r=-1;
                        }
                }
        return r;
        }




void Interface::show_splash_screen()
        {
        splashScreen = new SplashScreen(RESOURCES_DIR "/protuxSplash.jpg");
        }


void Interface::hide_splash_screen() // deprecated
        {
        }


void Interface::help()
        {
        PENTER;
        if (isHelpHidden)
                {
                helpWindow->show();
                isHelpHidden=false;
                }
        else
                {
                helpWindow->close();
                isHelpHidden=true;
                }
        PEXIT;
        }

void Interface::quick_help()
        {
        PENTER;
        PEXIT;
        }

void Interface::show_jmb_map()
        {
        PENTER;
        PEXIT;
        }

void Interface::about_protux()
        {
        PENTER;
        QString authors="The Protux Team : Luciano Giordana, Reinhard Amersberger, Tapio Laxstrom, David Bain, Nathan Langford, Fabio dos Santos, Martin Herren, Eduardo Pimenta, Valery Beluntsov, Marcio Enrico, Remon Sijrier, and all the people from Free Software world, who made important technologies where Protux is based on ( Gcc, QT, KDE, XFree, Linux, and so on...)                                              ";
        infoLcd->roll_message(authors,80,2);
        show_splash_screen();
        PEXIT;
        }

MustuxLcd* Interface::get_default_lcd()
        {
        return infoLcd;
        }


void Interface::update_properties()
        {
        PENTER;
        jmb->set_clear_time(GlobalProperties::get_int("JMB_CLEAR_TIME"));
        jmb->set_hold_sensitiveness(GlobalProperties::get_int("JMB_HOLD_TIMEOUT"));
        jmb->set_double_fact_interval(GlobalProperties::get_int("JMB_DOUBLE_FACT_TIMEOUT"));
        PEXIT;
        }


void Interface::info(QString message)
        {
        defaultLcd->print(message,0);
        }

void Interface::info(QString message, int line)
        {
        defaultLcd->print(message,line);
        }

BusMonitor* Interface::get_bus_monitor()
        {
        return busMonitor;
        }


void Interface::set_current_project(Project *pCurrentProject)
        {
	PENTER2;
        currentProject = pCurrentProject;
        mainVBoxLayout->addWidget(currentProject);
        mainVBoxLayout->activate();
	currentProject->set_output_spectrogram(spectrogram);
        currentProject->show();
	PEXIT2;
        }

Project* Interface::get_current_project()
        {
        return currentProject;
        }

ProjectManager* Interface::get_project_manager()
        {
        return projectManager;
        }


void Interface::update_system_resources()
        {
        if ((currentProject) && (currentProject->get_current_song()->is_snap_on()))
                snapStatus->setText("SNAP ON");

        struct statfs fs;
        statfs("/", &fs);
        float space = (float) fs.f_bfree * fs.f_bsize / 1048576;
        QString s; s.setNum(space,'f',2);
        diskSpace->setText(s+" Mb");

        /*
        int statfs(const char *path, struct statfs *buf);
        DESCRIPTION
       statfs  returns  information  about a mounted file system.  path is the
       path name of any file within the mounted filesystem.  buf is a  pointer
       to a statfs structure defined as follows:

              struct statfs {
                 long    f_type;     // type of filesystem (see below)
                 long    f_bsize;    // optimal transfer block size
                 long    f_blocks;   // total data blocks in file system
                 long    f_bfree;    // free blocks in fs
                 long    f_bavail;   // free blocks avail to non-superuser
                 long    f_files;    // total file nodes in file system
                 long    f_ffree;    // free file nodes in fs
                 fsid_t  f_fsid;     // file system id
                 long    f_namelen;  // maximum length of filenames
                 long    f_spare[6]; // spare for later
              };
        */
        }


// EOF

