/**************************************************************************************

        PROTUX - THE FREE PROFESSIONAL AUDIO TOOLS FOR LINUX
        AUTHOR : See AUTHORS file for details

        This software is distributed under the terms of the GNU General Public License
        as specified in the COPYING file.

***************************************************************************************/
#include <stdio.h>
#include <math.h>
#include <qwidget.h>
#include <mustux.h>
#include "Audio.hh"
#include "Peak.hh"
#include <qtimer.h>

// TODO : Sometimes the indexpointer of tempRAMBuffer exceeds the size of the array only with recorded audio.
//        This is some problem with the audioSource totalAudioBlocks??
// TODO : Make peak building/saving/loading proces independend of bit-depth. It now only takes care of 16 bits audio.

const int Peak::MAX_ZOOM_USING_SOURCEFILE = 8;
const int Peak::zoomStep[MAX_ZOOM_LEVELS] = {
                                                1,    2,    4,     6,     8,    12,    16,    20,
                                                24,   32,   48,    64,    96,   128,   192,
                                                256,  384,  512,   768,  1024,  1536,  2048,  3072,
                                        4096, 6144, 8192, 12288, 16384, 24576, 32768, 65536
                                        };


Peak::Peak()
        {
        PENTERCONS;
        PEXITCONS;
        }


Peak::~Peak()
        {
        PENTERDES;
        PEXITDES;
        }



int Peak::build(Audio* anAudioSource, QProgressBar *monitorProgressBar )
        {
        PENTER;
        audioSource = anAudioSource;
        MustuxAudioFileFormat* file = audioSource->file;
        file->rewind_audio();
        int channels = file->channels;
        int blockSize = file->blockSize;
        int headerSize = file->headerSize;
        long long totalBlocks = file->totalBlocks;

        static const int MAX_CHANNELS=2; // FIX ME
        long long posInLevel [ MAX_ZOOM_LEVELS ];

        //important: do not change tempStep into a value other then: 0, 1, 2, 4, 6. The lower this value,
        //the more time is needed for the build process, but it is done more accurately (which doesn't
        //make sense for the waveform-drawing as the monitor-resolution is the limiting factor in this case).
        int tempStep=6;
        totalRAMBufferSize=0;

        for (int hzoom = 0; hzoom<MAX_ZOOM_LEVELS; hzoom++)
                {
                RAMBuffer[hzoom] = (char*) 0;
                RAMBufferSize[hzoom]=0;
                }

        for (int hzoom=MAX_ZOOM_USING_SOURCEFILE+1;   hzoom<MAX_ZOOM_LEVELS;  hzoom++)
                {
                long long howManyZoomStepsForThisZoomLevel = totalBlocks / zoomStep[hzoom];
                RAMBufferSize[hzoom] = (howManyZoomStepsForThisZoomLevel + 1) * channels;
                RAMBuffer[hzoom] = new char[RAMBufferSize[hzoom]];
                totalRAMBufferSize += RAMBufferSize[hzoom];
                posInLevel[hzoom]=0;
                PMESG3("Created RAM buffer for HZOOM = %d of %d shorts ", hzoom , RAMBufferSize[hzoom] );
                }

        tempRAMBuffer = (char*) 0;
        long howManyZoomSteps = totalBlocks / zoomStep[tempStep];
        tempRAMBuffer = new char[((howManyZoomSteps + 1) * channels)];

        // try to load a previously saved image before rescan the file...
        if (load()>0)
                {
                PMESG("Loaded peaks from previously saved image");
                PEXIT;
                return 1;
                }

        QString afn = audioSource->get_pure_filename();
        long long blockPosCounter = 0;
        int ch = 0;
        long long scanningBlockPos = headerSize;
        long long stepInBlocks = zoomStep[tempStep];
        char* p = tempRAMBuffer;
        short sho;
        float f;
        // No saved peak. So it needs to re-scan the file
        MustuxLcd* lcd = audioSource->parentSong->get_default_lcd();
        lcd->print("Rescanning Peaks :   0  %", 3);
        PMESG("Rescanning peaks ... ");
        qApp->processEvents();
        //read peak information into tempRAMBuffer.
        do
                {
                // FIX ME ( ogg files and this next block_seek call)
                // this block_seek is positioning the files always in position 0
                // no matter what the scanningBlockPos value is
                // It is strange since in ogg seeking test, it worked fine
                int r = file->block_seek(scanningBlockPos);
                //printf("scanningBlockPos=%d   currentPos=%d\n",channels,scanningBlockPos,(int)file->block_tell());
                for  (ch=0; ch<channels; ch++)
                        {
                        file->read_fragment((char*)&sho, sizeof(short));
                        if (sho < 0) sho *= -1;
                        *(p++) = sho / 256;
                        }
                blockPosCounter++;
                scanningBlockPos+=stepInBlocks;

                if (blockPosCounter % 50000==0)
                        {
                        f = 60.0 * (float) (blockPosCounter * zoomStep[tempStep]) / totalBlocks;
                        int i = (int) f;
                        QString si; si.setNum(i);
                        lcd->print(si , 3,  20, false);
                        if (monitorProgressBar) monitorProgressBar->setProgress(i);
                        qApp->processEvents();
                        }
                }
        while ((blockPosCounter < (totalBlocks/zoomStep[tempStep])) && !file->eof());

        int step;
        int posInTempBuffer;
        int matchedZoomLevel;
        char representativeSample;
        char val;
        f = 60;
        //process the first 4 zoomlevels > MAX_ZOOM_USING_SOURCEFILE and place the values from tempRAMBuffer into RAMBUffer
        for (int hzoom=MAX_ZOOM_USING_SOURCEFILE+1; hzoom<13; hzoom++)
                {
                blockPosCounter = 0;
                step = 0;
                do
                        {
                        for  (ch=0; ch<channels; ch++)
                                {
                                representativeSample = 0;
                                matchedZoomLevel=hzoom;
                                posInTempBuffer = step;
                                for(int i=0; i<(zoomStep[hzoom]/zoomStep[tempStep]); i++)
                                        {
                                        if ((posInTempBuffer < (howManyZoomSteps * channels)))
                                                {
                                                val = tempRAMBuffer[posInTempBuffer];
                                                if (val > representativeSample) representativeSample = val;
                                                posInTempBuffer += channels;
                                                }
                                        }
                                long long pl = posInLevel [ matchedZoomLevel ];
                                RAMBuffer [ matchedZoomLevel ] [ pl ] = representativeSample;
                                posInLevel [ matchedZoomLevel ]++;
                                step++;
                                }
                        step += ((zoomStep[hzoom] * channels) / zoomStep[tempStep]) - channels;
                        blockPosCounter++;
                        }
                while (blockPosCounter < (totalBlocks / zoomStep[hzoom]));
                f += 1.8181;
                int i = (int) f;
                QString si; si.setNum(i);
                lcd->print(si , 3,  20, false);
                if (monitorProgressBar) monitorProgressBar->setProgress(i);
                //qApp->processEvents();
                }

        tempStep = 11;
        //process the remaining higher zoomlevels, but use values from zoomStep[11] which are already in RAMBuffer.
        //this step calculates 18 zoomlevels, and by using the RAMBuffer[zoomStep[11]] instead of tempRAMBuffer,
        //it reduces processing time for these zoomlevels at least 4 times.
        for (int hzoom=13; hzoom<MAX_ZOOM_LEVELS; hzoom++)
                {
                blockPosCounter = 0;
                step = 0;
                do
                        {
                        for  (ch=0; ch<channels; ch++)
                                {
                                representativeSample = 0;
                                matchedZoomLevel=hzoom;
                                posInTempBuffer = step;
                                for(int i=0; i<(zoomStep[hzoom]/zoomStep[tempStep]); i++)
                                        {
                                        if ((posInTempBuffer < (howManyZoomSteps * channels)))
                                                {
                                                val = RAMBuffer [tempStep] [posInTempBuffer];
                                                if (val > representativeSample) representativeSample = val;
                                                posInTempBuffer += channels;
                                                }
                                        }
                                long long pl = posInLevel [ matchedZoomLevel ];
                                RAMBuffer [ matchedZoomLevel ] [ pl ] = representativeSample;
                                posInLevel [ matchedZoomLevel ]++;
                                step++;
                                }
                        step += ((zoomStep[hzoom] * channels) / zoomStep [tempStep]) - channels;
                        blockPosCounter++;
                        }
                while (blockPosCounter < (totalBlocks / zoomStep[hzoom]));
                f += 1.8181;
                int i = (int) f;
                QString si; si.setNum(i);
                lcd->print(si , 3,  20, false);
                if (monitorProgressBar) monitorProgressBar->setProgress(i);
                //qApp->processEvents();
                }


        delete tempRAMBuffer;

        lcd->print("Rescanning Peaks : 100 % .." , 3);
        if (monitorProgressBar) monitorProgressBar->setProgress(100);
        lcd->print("Rescanning Completed." , 4 );
        PMESG("Rescanning Peaks : Completed.");
        if (save()<0)
                {
                PMESG("WARNING : Cannot save Peak image (RAM Buffers) to a peak file");
                }
        PEXIT;
        return 1;
        }



int Peak::unbuild()
        {
        audioSourceFile->close_file();
        for (int i=0; i<MAX_ZOOM_LEVELS; i++)
                if (RAMBuffer[i])
                        delete RAMBuffer[i];
        return 1;
        }



int Peak::fill_buffer(char* bufferToFill, long long startBlock, int numBlocksToFill,  int hzoom)
        {
        MustuxAudioFileFormat* file = audioSource->file;
        int blockSize = file->blockSize;
        int channels = file->channels;
        int headerSize = file->headerSize;
        //PDEBUG("blockSize=%d headerSize=%d",blockSize,headerSize);
        int blocksFilled;
        if (hzoom > MAX_ZOOM_USING_SOURCEFILE)
                {
                long long offsetInBytes = (startBlock / zoomStep[hzoom]) * blockSize;
                long long offsetInShorts = offsetInBytes / sizeof(short) ;
                char* pi = RAMBuffer[hzoom] + offsetInShorts;
                char* pf = RAMBuffer[hzoom] + RAMBufferSize[hzoom];
                long long numShortsToFill = numBlocksToFill * channels;
                if (numShortsToFill >= (pf-pi))
                        numShortsToFill = pf-pi;
                if (numShortsToFill<0)
                        return 0;
                memcpy(bufferToFill, pi, numShortsToFill*sizeof(char));
                blocksFilled = numShortsToFill / channels;
                }
        else
                {
                long long stepInBytes = blockSize * zoomStep[hzoom];
                long long pos = startBlock * blockSize + headerSize;
                long long originalPos = file->byte_tell();
                blocksFilled = 0;
                short sho;
                char* p = bufferToFill;
                do
                        {
                        file->byte_seek(pos);
                        for (int ch=0; ch < channels; ch++)
                                {
                                file->read_fragment((char *)&sho, sizeof(short));
                                *(p++)=sho / 256;
                                }
                        blocksFilled++;
                        pos+=stepInBytes;
                        }
                while ((blocksFilled < numBlocksToFill) && (!file->eof()));
                file->byte_seek(originalPos);
                }
        return blocksFilled;
        }


void Peak::show_info()
        {
        PMESG("Peakfiles information\n");
        //TODO
        }



int Peak::save()
        {
        PENTER;
        QString peakFilename = audioSource->rootDir + "/"+audioSource->get_pure_filename() + ".peak";
        PMESG("Saving Peak to %s", (const char*) peakFilename.ascii());
        size_t size ;
        FILE* file = fopen((const char*)peakFilename.ascii(),"w+");
        char* buf;

        if (!file)
                {
                PERROR("Cannot save Peak image to %s", (const char*) peakFilename.ascii());
                PEXIT;
                return -1;
                }
        //store the values from RAMBuffer in a (temp) buf, and write buf at ones to the hard disk, delete buf afterwards.
        buf = (char*) malloc (totalRAMBufferSize*sizeof(char));
        int pos=0;
        for (int hzoom = MAX_ZOOM_USING_SOURCEFILE+1; hzoom<MAX_ZOOM_LEVELS; hzoom++)
                {
                for (int j=0; j<RAMBufferSize[hzoom]; j++)
                        {
                        char sho = RAMBuffer[hzoom][j];
                        buf[pos] = sho;
                        pos++;
                        }
                }
        fwrite(buf,sizeof(char),totalRAMBufferSize,file);
        PMESG("RAM Buffers saved. Total size of file is %d",totalRAMBufferSize * sizeof(char) );
        fclose(file);
        delete buf;
        PEXIT;
        return 1;
        }



int Peak::load()
        {
        PENTER;
        QString peakFilename = audioSource->rootDir + "/"+ audioSource->get_pure_filename() + ".peak";
        PMESG("Looking for peak image file %s", (const char*) peakFilename.ascii());
        FILE* pfile = fopen((const char*)peakFilename.ascii(),"r+");
        char* buf;
        if (!pfile)
                {
                PEXIT;
                return -1;
                }
        PMESG("Found it ! Loading it into RAM Buffers... ");
        //reading peakfile into (temp) buf, so the file reading can be done in one big slurp.
        buf = (char*) malloc (totalRAMBufferSize*sizeof(char));
        fread(buf,sizeof(char),totalRAMBufferSize,pfile);
        int pos = 0;
        //putting the values in the right manner into RAMBuffer, deleting buf afterwards
        for (int hzoom = MAX_ZOOM_USING_SOURCEFILE+1; hzoom<MAX_ZOOM_LEVELS; hzoom++)
                {
                for (int j=0; j<RAMBufferSize[hzoom]; j++)
                        {
                        RAMBuffer[hzoom][j] = buf[pos];
                        pos++;
                        }
                }
        PMESG("RAM Buffers sucessfully loaded.");
        fclose(pfile);
        delete buf;
        PEXIT;
        return 1;
        }


// eof
