/**************************************************************************************

        PROTUX - THE FREE PROFESSIONAL AUDIO TOOLS FOR LINUX
        AUTHOR : See AUTHORS file for details

        This software is distributed under the terms of the GNU General Public License
        as specified in the COPYING file.

***************************************************************************************/

#include "Project.hh"
#include <qtabwidget.h>
#include <qmessagebox.h>
#include <qfile.h>
#include <qtextstream.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <stdlib.h>

Project::Project(QString pTitle, Interface* pParentInterface)
        : QWidget( (QWidget*) pParentInterface)
        {
        PENTERCONS;
        parentInterface = pParentInterface;
        resize(500,400);
        title = pTitle;
        QBoxLayout* mainBox = new QVBoxLayout( this, 1);

        mainBox->setResizeMode(QLayout::FreeResize);

        songsContainer = new QTabWidget(this);

        songsContainer->setFocusPolicy(QWidget::NoFocus);
        mainBox->addWidget(songsContainer);
        connect( songsContainer, SIGNAL( currentChanged(QWidget*) ), this, SLOT(tab_changed()));

        mainBox->activate();
        userDir = QString(getenv("HOME"));

        projsDir = userDir + "/protuxprojects";
        rootDir = projsDir + "/" + title;
        
        for (int i=0; i<MAX_SONGS_PER_PROJECT; i++)
                song[i]=(Song*) 0;
                
        PEXITCONS;
        }

Project::~Project()
        {
        PENTERDES;
        for (int i=0; i<numSongs; i++)
                delete song[i];
        PEXITDES;
        }

int Project::create(int pNumSongs)
        {
        PENTER;
        PMESG("Creating new project %s  NumSongs=%d",(const char*)title.latin1(),pNumSongs);
        FILE* f;
        char* data;
        QString sdata;

        if (mkdir((const char*)rootDir.latin1(), S_IREAD | S_IWRITE | S_IEXEC)<0)
                {
                PERROR("Cannot create dir %s", (const char*) rootDir.latin1());
                PEXIT;
                return -1;
                }

        QString sfilename = rootDir + "/project.properties";
        char* filename = (char*) sfilename.latin1();
        f = fopen(filename,"w");
        if (!f)
                {
                PERROR("Cannot open %s for write", filename);
                PEXIT;
                return -1;
                }


        sdata = "Title=" + title + "\n";
        data = (char*) sdata.latin1();
        fwrite( data, 1, sdata.length(), f );

        sdata.setNum(pNumSongs);
        sdata="NumSongs="+sdata+"\n";
        data = (char*) sdata.latin1();
        fwrite( data, 1, sdata.length(), f );

        sdata = "Engineer=" + engineer + "\n";
        data = (char*) sdata.latin1();
        fwrite( data, 1, sdata.length(), f );

        fclose(f);

        numSongs=0; // I have to zero this because each add_song will increase it properly
        for (int i=0; i< pNumSongs; i++)
                {
                QWidget* tabPage =  new QWidget(songsContainer);
                QString s; s.setNum(i+1);
                QString tabLabel = "SONG "+s;
                songsContainer->insertTab(tabPage,tabLabel);
                QBoxLayout* songBox = new QVBoxLayout( tabPage, 1);
                songBox->setResizeMode(QLayout::FreeResize);
                int rate = GlobalProperties::get_int("DEFAULT_SAMPLE_RATE");
                int bitDepth = GlobalProperties::get_int("DEFAULT_BIT_DEPTH");
                song[i] = new Song(numSongs+1, this, tabPage , rate, bitDepth);
                songBox->addWidget( song[i] );
                song[i]->build(GlobalProperties::get_int("DEFAULT_NUM_TRACKS"));
                numSongs++;
                set_current_song(i);
                }
        parentInterface->info("NEW PROJECT CREATED");
        PEXIT;
        return 1;
        }


int Project::load() // try to load the project by its title
        {
        PENTER;
        // check if the project exists
        int r=0;
        QString sNumSongs;
        QFile f(rootDir+"/project.properties");
        if ( f.open(IO_ReadOnly) )
                {
                QTextStream t( &f );
                title = t.readLine();
                sNumSongs = t.readLine();
                engineer = t.readLine();
                f.close();
                PMESG("Loading project %s  NumSongs=%s",(const char*)title.latin1(),(const char*)sNumSongs.latin1());
                int pNumSongs = sNumSongs.toInt();

                numSongs=0;
                for (int i=0; i< pNumSongs; i++)
                        {
                        QWidget* tabPage =  new QWidget(songsContainer);
                        QString s; s.setNum(i+1);
                        QString tabLabel = "SONG "+s;
                        songsContainer->insertTab(tabPage,tabLabel);

                        QBoxLayout* songBox = new QVBoxLayout( tabPage, 1);
                        songBox->setResizeMode(QLayout::FreeResize);

                        int rate = 44100; // TODO read from song_`i`/song.properties
                        int bitDepth = 16; // TODO read from song_`i`/song.properties
                        song[i] = new Song(numSongs+1, this, tabPage , rate, bitDepth);
                        songBox->addWidget( song[i] );
                        set_current_song(i);
                        numSongs++;
                        }

                for (int i=0; i<numSongs; i++)
                        song[i]->load();

                parentInterface->info("PROJECT " + title + " LOADED");
                r=1;
                }
        else
                {
                parentInterface->info("PROJECT " + title + " CANNOT BE LOADED !!");
                r=-1;
                }
        PEXIT;
        return r;
        }




void Project::set_title(QString pTitle)
        {
        title = pTitle;
        }

void Project::set_engineer(QString pEngineer)
        {
        engineer=pEngineer;
        }

QString Project::get_title()
        {
        return title;
        }

int Project::get_number_of_songs()
        {
        return numSongs;
        }


QString Project::get_engineer()
        {
        return engineer;
        }

Song* Project::add_song()
        {
        return add_song(GlobalProperties::get_int("DEFAULT_SAMPLE_RATE"),
                        GlobalProperties::get_int("DEFAULT_BIT_DEPTH"));
        }

Song* Project::add_song(int rate, int bitDepth)
        {
        PENTER;
        if (numSongs==MAX_SONGS_PER_PROJECT)
                 {
                 PEXIT;
                 return (Song*) 0;
                 }
        QWidget* tabPage =  new QWidget(songsContainer);
        QString s; s.setNum(numSongs+1);
        QString tabLabel = "SONG "+s;
        songsContainer->insertTab(tabPage,tabLabel);

        QBoxLayout* songBox = new QVBoxLayout( tabPage, 1);
        songBox->setResizeMode(QLayout::FreeResize);

        song[numSongs] = new Song(numSongs+1, this, tabPage , rate, bitDepth);
        songBox->addWidget( song[numSongs] );

        set_current_song(numSongs);
        numSongs++;
        PEXIT;
        return (Song*) 0;
        }


int Project::set_current_song(Song* pSong)
        {
        PENTER;
        if (!pSong)
                 {
                 PERROR("Internal Error : Trying to set current song : invalid Song object reference" );
                 }
        int res=-1;
        for (int i=0; i<numSongs; i++)
                 {
                 if (pSong==song[i])
                      {
                      res=i;
                      currentSongIndex=i;
                      break;
                      }
                 }
        PEXIT;
        return res;
        }


void Project::set_current_song(int index)
        {
        if ((index<0) || (index>MAX_SONGS_PER_PROJECT) || (index>numSongs))
                 {
                 PERROR("Internal Error : Trying to set current song : invalid index %d",index );
                 }
        else
                {
                currentSongIndex=index;
                songsContainer->setCurrentPage(currentSongIndex);
                }
        }



int Project::get_current_song_index()
        {
        return currentSongIndex;
        }


Song* Project::get_current_song()
        {
        return song[currentSongIndex];
        }


int Project::save()
        {
        PENTER;
        FILE* f;
        char* data;
        QString sdata;

        mkdir(rootDir, S_IREAD | S_IWRITE | S_IEXEC);
        QString sfilename = rootDir + "/project.properties";
        char* filename = (char*) sfilename.latin1();
        f = fopen(filename,"w");
        if (!f)
                {
                PERROR("Cannot open %s for write", filename);
                PEXIT;
                return -1;
                }

        sdata = title + "\n";
        data = (char*) sdata.latin1();
        fwrite( data, 1, sdata.length(), f );

        sdata.setNum(numSongs);
        sdata=sdata+"\n";
        data = (char*) sdata.latin1();
        fwrite( data, 1, sdata.length(), f );

        sdata = engineer + "\n";
        data = (char*) sdata.latin1();
        fwrite( data, 1, sdata.length(), f );

        fclose(f);

        for (int i=0; i< numSongs; i++)
                {
                song[i]->save();
                }
        PEXIT;
        return 0;
        }


QString Project::get_capture_dir()
        {
        return song[currentSongIndex]->get_capture_dir();
        }


QString Project::get_root_dir()
        {
        return rootDir;
        }

int Project::rename()
    {
    return 0;
    }


int Project::render()
        {
        // TODO
        return 0;
        }

void Project::show_info()
        {
// FOR LATER       parentInterface->get_default_lcd()->draw_string("CURRENT PROJECT : blabla",400,10);
// FOR LATER       parentInterface->get_default_lcd()->draw_string("CURRENT SONG : 02 - blabla",400,30);
        }


int Project::import_audio(QString filename,QProgressBar *monitorProgressBar )
        {
        PENTER;
        int r = song[currentSongIndex]->import_audio(filename,monitorProgressBar);
        PEXIT;
        return r;
        }

Interface* Project::get_parent_interface()
        {
        return parentInterface;
        }

void Project::tab_changed()
        {
        currentSongIndex = songsContainer->currentPageIndex();
        }


void Project::set_output_spectrogram(Spectrogram* pSpectrogram)
        {
        spectrogram=pSpectrogram;
        for (int i=0; i< numSongs; i++)
                {
                song[i]->set_output_spectrogram(spectrogram);
                }

        }

MustuxLcd* Project::get_default_lcd()
        {
        return parentInterface->get_default_lcd();
        }

BusMonitor* Project::get_bus_monitor()
        {
        return parentInterface->get_bus_monitor();
        }

//eof

