/**************************************************************************************

        PROTUX - THE FREE PROFESSIONAL AUDIO TOOLS FOR LINUX
        AUTHOR : See AUTHORS file for details

        This software is distributed under the terms of the GNU General Public License
        as specified in the COPYING file.

***************************************************************************************/
#include "ProjectManager.hh"

#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include <qvariant.h>
#include <qheader.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qimage.h>
#include <qpixmap.h>

#include <mustux.h>

#include "FileBrowser.hh"
#include "DevelGuardian.hh"
#include "Interface.hh"
#include "Mixer.hh"

ProjectManager::ProjectManager( Interface* pParentInterface,  const char* name ) : QDialog( (QWidget*) 0, name )
        {
        PENTERCONS;
        parentInterface = pParentInterface;

        if ( !name )
                setName( "ProjectManager" );
        resize( 780, 500 );
        setMaximumSize( QSize( 1024, 768 ) );
        setCaption( "<PM> Project Manager" );

        mainTabPane = new QTabWidget( this );
        mainTabPane->setGeometry( QRect( 0, 0, 780, 500 ) );
        connect( mainTabPane, SIGNAL( currentChanged(QWidget*) ), this, SLOT(tab_changed()));

        // Album Project tab -------------------------------------------------
        tab = new QWidget( mainTabPane);
        QWidget* privateLayoutWidget = new QWidget( tab );
        privateLayoutWidget->setGeometry( QRect( 10, 300, 710, 24 ) );
        QHBoxLayout* hlayout1 = new QHBoxLayout( privateLayoutWidget, 0, 6);
        lblAlbumTitle = new QLabel( privateLayoutWidget);
        lblAlbumTitle->setMinimumSize( QSize( 65, 0 ) );
        lblAlbumTitle->setText( tr( "CURRENT PROJECT : Title" ) );
        hlayout1->addWidget( lblAlbumTitle );
        fieldAlbumTitle = new QLineEdit( privateLayoutWidget);
        fieldAlbumTitle->setText( tr( "" ) );
        hlayout1->addWidget( fieldAlbumTitle );
        QWidget* privateLayoutWidget_2 = new QWidget( tab);
        privateLayoutWidget_2->setGeometry( QRect( 10, 330, 710, 22 ) );
        QHBoxLayout* hlayout2 = new QHBoxLayout( privateLayoutWidget_2, 0, 6);
        lblNOfSongs = new QLabel( privateLayoutWidget_2);
        lblNOfSongs->setText( tr( "N.of Songs" ) );
        hlayout2->addWidget( lblNOfSongs );
        fieldNOfSongs = new QLineEdit( privateLayoutWidget_2);
        fieldNOfSongs->setMaximumSize(QSize(60,20));
        fieldNOfSongs->setEnabled( FALSE );
        fieldNOfSongs->setText( tr( "1" ) );
        hlayout2->addWidget( fieldNOfSongs );
        lblStatus = new QLabel( privateLayoutWidget_2);
        lblStatus->setText( tr( "Status" ) );
        hlayout2->addWidget( lblStatus );
        cmbStatus = new QComboBox( FALSE, privateLayoutWidget_2);
        cmbStatus->insertItem( tr( "Planning" ) );
        cmbStatus->insertItem( tr( "Rehearsals" ) );
        cmbStatus->insertItem( tr( "Recording" ) );
        cmbStatus->insertItem( tr( "Finishing" ) );
        cmbStatus->insertItem( tr( "Tournee" ) );
        hlayout2->addWidget( cmbStatus );
        lblTotalDur = new QLabel( privateLayoutWidget_2);
        lblTotalDur->setText( tr( "Dur: ???" ) );
        hlayout2->addWidget( lblTotalDur );
        QWidget* privateLayoutWidget_3 = new QWidget( tab);
        privateLayoutWidget_3->setGeometry( QRect( 10, 360, 710, 24 ) );
        QHBoxLayout* hlayout3 = new QHBoxLayout( privateLayoutWidget_3, 0, 6);
        lblEngineer = new QLabel( privateLayoutWidget_3);
        lblEngineer->setMinimumSize( QSize( 95, 0 ) );
        lblEngineer->setText( tr( "Protux Engineer" ) );
        hlayout3->addWidget( lblEngineer );
        fieldEngineer = new QLineEdit( privateLayoutWidget_3);
        hlayout3->addWidget( fieldEngineer );
        QWidget* privateLayoutWidget_4 = new QWidget( tab );
        privateLayoutWidget_4->setGeometry( QRect( 10, 420, 710, 24 ) );
        QHBoxLayout* hlayout4 = new QHBoxLayout( privateLayoutWidget_4, 0, 6);
        btnRender = new QPushButton( privateLayoutWidget_4);
        btnRender->setText( tr( "Render ..." ) );
        connect( btnRender, SIGNAL(clicked()), this, SLOT(btnRender_clicked()));
        //QToolTip::add( btnRender, tr( "Render all underendered elements. Do this before burning a CD" ) );
        hlayout4->addWidget( btnRender );
        btnMaster = new QPushButton( privateLayoutWidget_4);
        btnMaster->setEnabled( FALSE );
        btnMaster->setText( tr( "Master ..." ) );
        connect( btnMaster, SIGNAL(clicked()), this, SLOT(btnMaster_clicked()));
        //QToolTip::add( btnMaster, tr( "Opens the masterization dialog" ) );
        hlayout4->addWidget( btnMaster );
        btnBurnCD = new QPushButton( privateLayoutWidget_4);
        btnBurnCD->setEnabled( FALSE );
        connect( btnBurnCD, SIGNAL(clicked()), this, SLOT(btnBurnCD_clicked()));
        //QFont btnBurnCD_font(  btnBurnCD->font() );
        //btnBurnCD_font.setFamily( "Helvetica" );
        //btnBurnCD_font.setPointSize( 10 );
        //btnBurnCD_font.setBold( TRUE );
        //btnBurnCD->setFont( btnBurnCD_font );
        btnBurnCD->setText( tr( "Burn CD" ) );
        hlayout4->addWidget( btnBurnCD );
        QWidget* privateLayoutWidget_5 = new QWidget( tab );
        privateLayoutWidget_5->setGeometry( QRect( 10, 390, 710, 24 ) );
        QHBoxLayout* hlayout5 = new QHBoxLayout( privateLayoutWidget_5, 0, 6);
        btnCreateProject = new QPushButton( privateLayoutWidget_5 );
        btnCreateProject->setText( tr( "CREATE NEW" ) );
        hlayout5->addWidget( btnCreateProject );
        connect( btnCreateProject, SIGNAL(clicked()), this, SLOT(btnCreateProject_clicked()));
        btnLoadProject = new QPushButton( privateLayoutWidget_5 );
        btnLoadProject->setText( tr( "LOAD" ) );
        connect( btnLoadProject, SIGNAL(clicked()), this, SLOT(btnLoadProject_clicked()));
        //QToolTip::add( btnSaveProject, tr( "Load this album project (Save your current project first !!!)" ) );
        hlayout5->addWidget( btnLoadProject );
        btnSaveProject = new QPushButton( privateLayoutWidget_5 );
        btnSaveProject->setText( tr( "SAVE" ) );
        connect( btnSaveProject, SIGNAL(clicked()), this, SLOT(btnSaveProject_clicked()));
        //QToolTip::add( btnSaveProject, tr( "Save all Song projects under this album project" ) );
        hlayout5->addWidget( btnSaveProject );
        btnCopyToProject = new QPushButton( privateLayoutWidget_5 );
        btnCopyToProject->setText( tr( "Save as" ) );
        connect( btnCopyToProject, SIGNAL(clicked()), this, SLOT(btnCopyToProject_clicked()));
        //QToolTip::add( btnCopyToProject, tr( "Copy this project to another project" ) );
        hlayout5->addWidget( btnCopyToProject );
        fieldSaveProjAs = new QLineEdit( privateLayoutWidget_5 );
        hlayout5->addWidget( fieldSaveProjAs );
        btnDeleteProject = new QPushButton( privateLayoutWidget_5 );
        connect( btnDeleteProject, SIGNAL(clicked()), this, SLOT(btnDeleteProject_clicked()));
        btnDeleteProject->setText( tr( "DELETE" ) );
        //QToolTip::add( btnDeleteProject, tr( "Delete current project (DANGER)" ) );
        hlayout5->addWidget( btnDeleteProject );
        listAlbumProjects = new QListView( tab );
        listAlbumProjects->addColumn( tr( "Album Name" ) );
        listAlbumProjects->addColumn( tr( "N. of songs" ) );
        listAlbumProjects->addColumn( tr( "Space Alloc" ) );
        listAlbumProjects->addColumn( tr( "Status" ) );
        update_projects_list();
        listAlbumProjects->setGeometry( QRect( 10, 10, 710, 280 ) );
        connect(listAlbumProjects, SIGNAL(selectionChanged()), this, SLOT(listAlbumProject_projectSelected()));
        mainTabPane->insertTab( tab, tr( "Album Projects" ) );


        // Songs tab ----------------------------------------------------------
        tab_2 = new QWidget( mainTabPane );
        QWidget* privateLayoutWidget_6 = new QWidget( tab_2);
        privateLayoutWidget_6->setGeometry( QRect( 10, 430, 710, 22 ) );
        QHBoxLayout* hlayout6 = new QHBoxLayout( privateLayoutWidget_6, 0, 6);
        lblMasterGain = new QLabel( privateLayoutWidget_6 );
        lblMasterGain->setMinimumSize( QSize( 69, 0 ) );
        lblMasterGain->setText( tr( "Master Gain" ) );
        hlayout6->addWidget( lblMasterGain );
        fieldMasterGain = new QLineEdit( privateLayoutWidget_6 );
        fieldMasterGain->setText( tr( "0.0" ) );
        hlayout6->addWidget( fieldMasterGain );
        lblNumTracks = new QLabel( privateLayoutWidget_6 );
        lblNumTracks->setMinimumSize( QSize( 70, 0 ) );
        lblNumTracks->setText( tr( "Num. Tracks" ) );
        hlayout6->addWidget( lblNumTracks );
        fieldNumTracks = new QLineEdit( privateLayoutWidget_6);
        fieldNumTracks->setText( tr( "14" ) );
        fieldNumTracks->setEnabled(false);
        //QToolTip::add( fieldNumTracks, tr( "To add/remove tracks, use standard JMB actions , such <INSERT> or <DELETE>, right inside MTA" ) );
        hlayout6->addWidget( fieldNumTracks );
        lblSongDur = new QLabel( privateLayoutWidget_6);
        lblSongDur->setMinimumSize( QSize( 56, 0 ) );
        lblSongDur->setText( tr( "Duration" ) );
        lblSongDur->setScaledContents( FALSE );
        hlayout6->addWidget( lblSongDur );
        fieldSongDur = new QLineEdit( privateLayoutWidget_6 );
        fieldSongDur->setText( tr( "??" ) );
        fieldNumTracks->setEnabled(false);
        hlayout6->addWidget( fieldSongDur );
        QWidget* privateLayoutWidget_7 = new QWidget( tab_2);
        privateLayoutWidget_7->setGeometry( QRect( 10, 400, 710, 24 ) );
        QHBoxLayout* hlayout7 = new QHBoxLayout( privateLayoutWidget_7, 0, 6);
        lblArtist = new QLabel( privateLayoutWidget_7);
        lblArtist->setMinimumSize( QSize( 60, 0 ) );
        lblArtist->setText( tr( "Artists" ) );
        hlayout7->addWidget( lblArtist );
        fieldArtists = new QLineEdit( privateLayoutWidget_7);
        //fieldArtists->setText( tr( "Matt Bradley" ) );
        hlayout7->addWidget( fieldArtists );
        QWidget* privateLayoutWidget_8 = new QWidget( tab_2 );
        privateLayoutWidget_8->setGeometry( QRect( 10, 370, 710, 22 ) );
        QHBoxLayout* hlayout8 = new QHBoxLayout( privateLayoutWidget_8, 0, 6);
        TextLabel2 = new QLabel( privateLayoutWidget_8 );
        //TextLabel2->setText( tr( "Song : Au revoir" ) );
        hlayout8->addWidget( TextLabel2 );
        QWidget* privateLayoutWidget_9 = new QWidget( tab_2 );
        privateLayoutWidget_9->setGeometry( QRect( 10, 320, 710, 24 ) );
        QHBoxLayout* hlayout9 = new QHBoxLayout( privateLayoutWidget_9, 0, 6);
        btnSaveSong = new QPushButton( privateLayoutWidget_9 );
        connect(btnSaveSong, SIGNAL(clicked()), this, SLOT(btnSaveSong_clicked()));
        btnSaveSong->setMinimumSize( QSize( 80, 0 ) );
        btnSaveSong->setText( tr( "SAVE" ) );
        //QToolTip::add( btnSaveSong, tr( "Save this song project" ) );
        hlayout9->addWidget( btnSaveSong );
        btnSaveSongAs = new QPushButton( privateLayoutWidget_9);
        connect( btnSaveSongAs, SIGNAL(clicked()), this, SLOT(btnSaveSongAs_clicked()));
        btnSaveSongAs->setMinimumSize( QSize( 80, 0 ) );
        btnSaveSongAs->setText( tr( "Rename" ) );
        //QToolTip::add( btnSaveSongAs, tr( "Add a new song which is a copy of current song" ) );
        hlayout9->addWidget( btnSaveSongAs );
        lblOr = new QLabel( privateLayoutWidget_9 );
        lblOr->setText( tr( "or" ) );
        hlayout9->addWidget( lblOr );
        btn = new QPushButton( privateLayoutWidget_9 );
        btn->setMinimumSize( QSize( 80, 0 ) );
        btn->setText( tr( "COPY as" ) );
        connect( btn, SIGNAL(clicked()), this, SLOT(btnSaveSongCopyAs_clicked()));
        //QToolTip::add( btn, tr( "add an empty song project" ) );
        hlayout9->addWidget( btn );
        fieldCopyTo = new QLineEdit( privateLayoutWidget_9 );
        fieldCopyTo->setMinimumSize( QSize( 410, 0 ) );
        fieldCopyTo->setMaximumSize( QSize( 300, 32767 ) );
        fieldCopyTo->setText( tr( "Revoir (reprise)", "Record" ) );
        hlayout9->addWidget( fieldCopyTo );
        listSongs = new QListView( tab_2, "listSongs" );
        listSongs->addColumn( tr( "Song Name" ) );
        listSongs->addColumn( tr( "N. of Tracks" ) );
        listSongs->addColumn( tr( "Space Alloc" ) );
        listSongs->addColumn( tr( "Status" ) );
        QListViewItem * item = new QListViewItem( listSongs, 0 );
        item->setText( 0, tr( "Untitled Song" ) );
        listSongs->setGeometry( QRect( 10, 10, 710, 310 ) );
        QWidget* privateLayoutWidget_91 = new QWidget( tab_2 );
        privateLayoutWidget_91->setGeometry( QRect( 10, 350, 710, 30 ) );
        QHBoxLayout* hlayout10 = new QHBoxLayout( privateLayoutWidget_91, 0, 6);
        QPushButton* btnNewSong = new QPushButton( privateLayoutWidget_91);
        btnNewSong->setMinimumSize( QSize( 80, 25 ) );
        btnNewSong->setMaximumSize( QSize( 120, 25 ) );
        btnNewSong->setText( tr( "NEW" ) );
        //QToolTip::add( btnNewSong, tr( "Create a New Song" ) );
        hlayout10->addWidget( btnNewSong );
        cmbSongStatus = new QComboBox( FALSE, privateLayoutWidget_91 );
        cmbSongStatus->insertItem( tr( "Planning" ) );
        cmbSongStatus->insertItem( tr( "Rehearsals" ) );
        cmbSongStatus->insertItem( tr( "Recording" ) );
        cmbSongStatus->insertItem( tr( "Finishing" ) );
        cmbSongStatus->insertItem( tr( "Tournee" ) );
        cmbSongStatus->setMaximumSize( QSize( 94, 32767 ) );
        hlayout10->addWidget( cmbSongStatus );
        mainTabPane->insertTab( tab_2, tr( "Songs" ) );


        // Audio Sources tab -----------------------------------------------
        tab_3 = new QWidget( mainTabPane);
        QVBoxLayout* audioSourcesViewerLayout = new QVBoxLayout( tab_3, 11, 6, "audioSourcesViewerLayout");

        audioListView = new QListView( tab_3 );
        audioSourcesViewerLayout->addWidget( audioListView );
        audioListView->addColumn( tr( "Name" ) );
        audioListView->addColumn( tr( "Type" ) );
        audioListView->addColumn( tr( "Lenght" ) );
        audioListView->addColumn( tr( "Size" ) );
        audioListView->addColumn( tr( "Clips" ) );
        audioListView->addColumn( tr( "Filters" ) );
        audioListView->addColumn( tr( "Rate" ) );
        audioListView->addColumn( tr( "Channels" ) );
        audioListView->addColumn( tr( "Bit Depth" ) );
        audioListView->addColumn( tr( "Project" ) );
        audioListView->addColumn( tr( "Group" ) );
        audioListView->addColumn( tr( "Hardware" ) );
        audioListView->addColumn( tr( "Description" ) );

        QWidget* audioSourcesButtonsContainer = new QWidget( tab_3 ); //        audioSourcesButtonsContainer->setGeometry( QRect( 10, 310, 700, 24 ) );
        audioSourcesViewerLayout->addWidget( audioSourcesButtonsContainer );
        QHBoxLayout* hlayout11 = new QHBoxLayout( audioSourcesButtonsContainer, 0, 6);
        btnDeleteSource = new QPushButton( audioSourcesButtonsContainer);
        btnDeleteSource->setText( tr( "Delete" ) );
        hlayout11->addWidget( btnDeleteSource );
        btnRemoveAllClips = new QPushButton( audioSourcesButtonsContainer);
        btnRemoveAllClips->setText( tr( "Remove All Clips" ) );
        hlayout11->addWidget( btnRemoveAllClips );
        btnExportSource = new QPushButton( audioSourcesButtonsContainer);
        btnExportSource->setText( tr( "Export ..." ) );
        hlayout11->addWidget( btnExportSource );

        grpPreListenSource = new QGroupBox( tab_3);
        audioSourcesViewerLayout->addWidget( grpPreListenSource );
        grpPreListenSource->setGeometry( QRect( 10, 340, 700, 110 ) );
        grpPreListenSource->setTitle( tr( "Listen" ) );
        btnSourcePlay = new QPushButton( grpPreListenSource);
        btnSourcePlay->setGeometry( QRect( 10, 20, 260, 22 ) );
        btnSourcePlay->setText( tr( "Play" ) );
        fieldAudioSourceInfo = new QLineEdit( grpPreListenSource );
        fieldAudioSourceInfo->setGeometry( QRect( 280, 20, 260, 70 ) );
        lblSourceSpectro = new QLabel( grpPreListenSource );
        lblSourceSpectro->setGeometry( QRect( 580, 40, 80, 15 ) );
        lblSourceSpectro->setMaximumSize( QSize( 32767, 15 ) );
        lblSourceSpectro->setText( tr( "FFT & Peak" ) );

        QWidget* privateLayoutWidget_11 = new QWidget( grpPreListenSource );
        privateLayoutWidget_11->setGeometry( QRect( 10, 50, 260, 16 ) );
        QHBoxLayout* hlayout12 = new QHBoxLayout( privateLayoutWidget_11, 0, 6);
        QWidget* privateLayoutWidget_12 = new QWidget( grpPreListenSource );
        privateLayoutWidget_12->setGeometry( QRect( 10, 80, 260, 18 ) );
        QHBoxLayout* hlayout13 = new QHBoxLayout( privateLayoutWidget_12, 0, 6);
        checkSourceInfo = new QCheckBox( privateLayoutWidget_12 );
        checkSourceInfo->setEnabled( TRUE );
        checkSourceInfo->setText( tr( "Auto Info" ) );
        hlayout13->addWidget( checkSourceInfo );
        checkSourceSpectro = new QCheckBox( privateLayoutWidget_12 );
        checkSourceSpectro->setText( tr( "AutoSpectro" ) );
        hlayout13->addWidget( checkSourceSpectro );

        // signals and slots connections
        connect( audioListView, SIGNAL( itemRenamed(QListViewItem*,int) ), this, SLOT( audioListView_itemRenamed(QListViewItem*,int) ) );
        connect( audioListView, SIGNAL( spacePressed(QListViewItem*) ), this, SLOT( audioListView_spacePressed(QListViewItem*) ) );
        connect( audioListView, SIGNAL( returnPressed(QListViewItem*) ), this, SLOT( audioListView_returnPressed(QListViewItem*) ) );
        connect( audioListView, SIGNAL( onItem(QListViewItem*) ), this, SLOT( audioListView_onItem(QListViewItem*) ) );
        connect( audioListView, SIGNAL( collapsed(QListViewItem*) ), this, SLOT( audioListView_collapsed(QListViewItem*) ) );
        connect( audioListView, SIGNAL( expanded(QListViewItem*) ), this, SLOT( audioListView_expanded(QListViewItem*) ) );
        connect( btnDeleteSource, SIGNAL(clicked()), this, SLOT(btnDeleteSource_clicked()));
        connect( btnRemoveAllClips, SIGNAL(clicked()), this, SLOT(btnRemoveAllClips_clicked()));
        connect( btnExportSource, SIGNAL(clicked()), this, SLOT(btnExportSource_clicked()));
        connect( btnSourcePlay, SIGNAL(clicked()), this, SLOT(btnSourcePlay_clicked()));

        mainTabPane->insertTab( tab_3, tr( "Audio Sources" ) );


        // File Browser ------------------------------------------------------
        tab_4 = new QWidget( mainTabPane);
        fileBrowser = new FileBrowser(tab_4);
        fileBrowser->setMinimumSize(QSize(710,260));
        grpPreListenFile = new QGroupBox( tab_4 );
        grpPreListenFile->setGeometry( QRect( 10, 340, 700, 110 ) );
        grpPreListenFile->setTitle( tr( "Pre-listen" ) );
        QWidget* privateLayoutWidget_13 = new QWidget( grpPreListenFile);
        privateLayoutWidget_13->setGeometry( QRect( 10, 50, 260, 16 ) );
        QHBoxLayout* hlayout14 = new QHBoxLayout( privateLayoutWidget_13, 0, 6);
        lblDSPVol2 = new QLabel( privateLayoutWidget_13 );
        lblDSPVol2->setText( tr( "Vol" ) );
        hlayout14->addWidget( lblDSPVol2 );
        sliderFile = new QSlider( privateLayoutWidget_13 );
        sliderFile->setMaximumSize( QSize( 32767, 14 ) );
        sliderFile->setOrientation( QSlider::Horizontal );
        hlayout14->addWidget( sliderFile );
        QWidget* privateLayoutWidget_14 = new QWidget( grpPreListenFile );
        privateLayoutWidget_14->setGeometry( QRect( 10, 80, 260, 18 ) );
        QHBoxLayout* hlayout15 = new QHBoxLayout( privateLayoutWidget_14, 0, 6);

        checkFileInfo = new QCheckBox( privateLayoutWidget_14 );
        checkFileInfo->setEnabled( TRUE );
        checkFileInfo->setText( tr( "Auto Info" ) );
        hlayout15->addWidget( checkFileInfo );

        checkFileSpectro = new QCheckBox( privateLayoutWidget_14 );
        checkFileSpectro->setText( tr( "AutoSpectro" ) );
        hlayout15->addWidget( checkFileSpectro );

        lblFileSpectro = new QLabel( grpPreListenFile );
        lblFileSpectro->setGeometry( QRect( 570, 60, 80, 15 ) );
        lblFileSpectro->setMaximumSize( QSize( 32767, 15 ) );
        lblFileSpectro->setText( tr( "FFT & Peak" ) );

        fieldFileInfo = new QLineEdit( grpPreListenFile );
        fieldFileInfo->setGeometry( QRect( 280, 20, 260, 70 ) );

        btnFilePlay = new QPushButton( grpPreListenFile );
        connect( btnFilePlay, SIGNAL(clicked()), this, SLOT(btnFilePlay_clicked()));
        btnFilePlay->setGeometry( QRect( 10, 20, 260, 22 ) );
        btnFilePlay->setText( tr( "Play" ) );

        grpImportFile = new QGroupBox( tab_4 );
        grpImportFile->setGeometry( QRect( 10, 260, 700, 80 ) );
        grpImportFile->setTitle( tr( "Import" ) );

        QWidget* privateLayoutWidget_15 = new QWidget( grpImportFile );
        privateLayoutWidget_15->setGeometry( QRect( 20, 20, 670, 24 ) );
        QHBoxLayout* hlayout16 = new QHBoxLayout( privateLayoutWidget_15, 0, 6);

        lblFileDestSong = new QLabel( privateLayoutWidget_15 );
        lblFileDestSong->setMaximumSize( QSize( 57, 32767 ) );
        lblFileDestSong->setText( tr( "Dest: Song" ) );
        hlayout16->addWidget( lblFileDestSong );

        fieldFileDestSong = new QLineEdit( privateLayoutWidget_15 );
        fieldFileDestSong->setMaximumSize( QSize( 25, 32767 ) );
        fieldFileDestSong->setText( tr( "1" ) );
        hlayout16->addWidget( fieldFileDestSong );

        lblFileDestTrack = new QLabel( privateLayoutWidget_15 );
        lblFileDestTrack->setMaximumSize( QSize( 36, 32767 ) );
        lblFileDestTrack->setText( tr( "Track" ) );
        hlayout16->addWidget( lblFileDestTrack );

        fieldFileDestTrack = new QLineEdit( privateLayoutWidget_15 );
        fieldFileDestTrack->setMaximumSize( QSize( 25, 32767 ) );
        fieldFileDestTrack->setText( tr( "1" ) );
        hlayout16->addWidget( fieldFileDestTrack );

        lblFileDestPos = new QLabel( privateLayoutWidget_15 );
        lblFileDestPos->setMaximumSize( QSize( 25, 32767 ) );
        lblFileDestPos->setText( tr( "Pos" ) );
        hlayout16->addWidget( lblFileDestPos );

        fieldFileDestPos = new QLineEdit( privateLayoutWidget_15 );
        fieldFileDestPos->setMinimumSize( QSize( 77, 0 ) );
        fieldFileDestPos->setMaximumSize( QSize( 55, 32767 ) );
        fieldFileDestPos->setText( tr( "0:02:30,14" ) );
        hlayout16->addWidget( fieldFileDestPos );

        btnAddFileAsSource = new QPushButton( privateLayoutWidget_15 );
        btnAddFileAsSource->setText( tr( "Add as Audio Source" ) );
        connect( btnAddFileAsSource, SIGNAL(clicked()), this, SLOT(btnAddFileAsSource_clicked()));
        //QToolTip::add( btnAddFileAsSource, tr( "Add this file/cd-track to song project as an audio source" ) );
        hlayout16->addWidget( btnAddFileAsSource );

        btnImportFileNow = new QPushButton( privateLayoutWidget_15 );
        btnImportFileNow->setText( tr( "Import now!" ) );
        connect( btnImportFileNow, SIGNAL(clicked()), this, SLOT(btnImportNow_clicked()));
        //QToolTip::add( btnImportFileNow, tr( "Add this file/cd-track to song project as an audio source" ) );
        hlayout16->addWidget( btnImportFileNow );

        QWidget* privateLayoutWidget_16 = new QWidget( grpImportFile );
        privateLayoutWidget_16->setGeometry( QRect( 20, 50, 670, 22 ) );
        QHBoxLayout* hlayout17 = new QHBoxLayout( privateLayoutWidget_16, 0, 6);

        pbarFileImport = new QProgressBar( privateLayoutWidget_16, "pbarFileImport" );
        pbarFileImport->setProgress( 0 );
        hlayout17->addWidget( pbarFileImport );

        checkFileFakeImport = new QCheckBox( privateLayoutWidget_16 );
        checkFileFakeImport->setText( tr( "Fake Import" ) );
        //QToolTip::add( checkFileFakeImport, tr( "Just import the audio source, spending the same amout of time, but does not put it into the song as audio source" ) );
        hlayout17->addWidget( checkFileFakeImport );
        mainTabPane->insertTab( tab_4, "<FB> File Broswer" );


        // CD BROWSER ---------------------------------------------------------
        tab_5 = new QWidget( mainTabPane, "tab_5" );
        lblCDBrowser_2 = new QLabel( tab_5 );
        lblCDBrowser_2->setGeometry( QRect( 290, 160, 120, 20 ) );
        lblCDBrowser_2->setText( tr( "Audio CD Browser" ) );
        grpImportCDA = new QGroupBox( tab_5 );
        grpImportCDA->setGeometry( QRect( 10, 260, 700, 80 ) );
        grpImportCDA->setTitle( tr( "Import" ) );
        QWidget* privateLayoutWidget_17 = new QWidget( grpImportCDA );
        privateLayoutWidget_17->setGeometry( QRect( 20, 20, 670, 24 ) );
        QHBoxLayout* hlayout18 = new QHBoxLayout( privateLayoutWidget_17, 0, 6);
        lblDestSongCDA = new QLabel( privateLayoutWidget_17 );
        lblDestSongCDA->setMaximumSize( QSize( 57, 32767 ) );
        lblDestSongCDA->setText( tr( "Dest: Song" ) );
        hlayout18->addWidget( lblDestSongCDA );
        fieldDestSongCDA = new QLineEdit( privateLayoutWidget_17 );
        fieldDestSongCDA->setMaximumSize( QSize( 25, 32767 ) );
        fieldDestSongCDA->setText( tr( "1" ) );
        hlayout18->addWidget( fieldDestSongCDA );
        lblDestTrackCDA = new QLabel( privateLayoutWidget_17 );
        lblDestTrackCDA->setMaximumSize( QSize( 36, 32767 ) );
        lblDestTrackCDA->setText( tr( "Track" ) );
        hlayout18->addWidget( lblDestTrackCDA );
        fieldDestTrackCDA = new QLineEdit( privateLayoutWidget_17 );
        fieldDestTrackCDA->setMaximumSize( QSize( 25, 32767 ) );
        fieldDestTrackCDA->setText( tr( "1" ) );
        hlayout18->addWidget( fieldDestTrackCDA );
        lblDestPosCDA = new QLabel( privateLayoutWidget_17 );
        lblDestPosCDA->setMaximumSize( QSize( 25, 32767 ) );
        lblDestPosCDA->setText( tr( "Pos" ) );
        hlayout18->addWidget( lblDestPosCDA );
        fieldDestPosCDA = new QLineEdit( privateLayoutWidget_17 );
        fieldDestPosCDA->setMinimumSize( QSize( 77, 0 ) );
        fieldDestPosCDA->setMaximumSize( QSize( 55, 32767 ) );
        fieldDestPosCDA->setText( "0:02:30,14" );
        hlayout18->addWidget( fieldDestPosCDA );
        btnAddSourceCDA = new QPushButton( privateLayoutWidget_17 );
        connect( btnAddSourceCDA, SIGNAL(clicked()), this, SLOT(btnAddSourceCDA_clicked()));
        btnAddSourceCDA->setText( tr( "Add as Audio Source" ) );
        //QToolTip::add( btnAddSourceCDA, tr( "Add this file/cd-track to song project as an audio source" ) );
        hlayout18->addWidget( btnAddSourceCDA );
        btnImportNowCDA = new QPushButton( privateLayoutWidget_17 );
        connect( btnImportNowCDA, SIGNAL(clicked()), this, SLOT(btnImportNowCDA_clicked()));
        btnImportNowCDA->setText( tr( "Import now!" ) );
        //QToolTip::add( btnImportNowCDA, tr( "Add this file/cd-track to song project as an audio source" ) );
        hlayout18->addWidget( btnImportNowCDA );
        QWidget* privateLayoutWidget_18 = new QWidget( grpImportCDA);
        privateLayoutWidget_18->setGeometry( QRect( 20, 50, 670, 22 ) );
        QHBoxLayout* hlayout19 = new QHBoxLayout( privateLayoutWidget_18, 0, 6);
        pbarImportCDA = new QProgressBar( privateLayoutWidget_18 );
        pbarImportCDA->setProgress( 0 );
        hlayout19->addWidget( pbarImportCDA );
        checkFakeImportCDA = new QCheckBox( privateLayoutWidget_18 );
        checkFakeImportCDA->setText( tr( "Fake Import" ) );
        //QToolTip::add( checkFakeImportCDA, tr( "Just import the audio source, spending the same amout of time, but does not put it into the song as audio source" ) );
        hlayout19->addWidget( checkFakeImportCDA );
        grpPreListenCDA = new QGroupBox( tab_5 );
        grpPreListenCDA->setGeometry( QRect( 10, 340, 700, 110 ) );
        grpPreListenCDA->setTitle( tr( "Pre-listen" ) );
        QWidget* privateLayoutWidget_19 = new QWidget( grpPreListenCDA);
        privateLayoutWidget_19->setGeometry( QRect( 10, 50, 260, 16 ) );
        QHBoxLayout* hlayout20 = new QHBoxLayout( privateLayoutWidget_19, 0, 6);
        lblCDAVol = new QLabel( privateLayoutWidget_19 );
        lblCDAVol->setText( tr( "Vol" ) );
        hlayout20->addWidget( lblCDAVol );
        sliderCDA = new QSlider( privateLayoutWidget_19 );
        sliderCDA->setMaximumSize( QSize( 32767, 14 ) );
        sliderCDA->setOrientation( QSlider::Horizontal );
        hlayout20->addWidget( sliderCDA );
        btnCDAPlay = new QPushButton( grpPreListenCDA );
        connect( btnCDAPlay, SIGNAL(clicked()), this, SLOT(btnCDAPlay_clicked()));
        btnCDAPlay->setGeometry( QRect( 10, 20, 260, 22 ) );
        btnCDAPlay->setText( tr( "Play" ) );
        mainTabPane->insertTab( tab_5, tr( "Audio CD Broswer" ) );


        mainTabPane->setCurrentPage ( 0 );
        currentProject = (Project*) 0;
        PEXITCONS;
        }

ProjectManager::~ProjectManager()
        {
        PENTERDES;
        PEXITDES;
        }

void ProjectManager::show()
        {
        QWidget::show();
        update();
        }

void ProjectManager::update()
        {
        PENTER;
        update_audio_sources_list();
        update_projects_list();
        PEXIT;
        }

void ProjectManager::update_audio_sources_list()
        {
        PENTER;
        Project* p = parentInterface->get_current_project();
        if (!p) {
                PEXIT;
                return;
                }
        Song* s = p->get_current_song();
        if (!s) {
                PEXIT;
                return;
                }
        AudioSourcesList* asl= s->get_asl();
        int tot = asl->get_total_sources();
        audioListView->clear();
        for (int i=0; i<tot; i++)
                {
                Audio* a = asl->get_source_for_index(i);
                if (!a) continue;
                QString sFile = a->file->filename;
                QString sType;
                if (sFile.find("wav")>=0)
                        sType = "WAV";
                else
                        sType = "PRAF";
                QString sRate; sRate.setNum(a->file->rate);
                QString sBitDepth; sBitDepth.setNum(a->file->bitDepth);
                QString sChannels = (a->file->channels==1?"MONO":"STEREO");
                QString sLenght; sLenght.setNum((double)a->file->totalBlocks,'f',0);
                QString sSize; sSize.setNum((double)a->file->fileSize,'f',0);
                QString sClips; sClips.setNum(a->get_clips_count());

                QListViewItem* it = new QListViewItem(audioListView);
                it->setText(0,sFile);
                it->setText(1,sType);
                it->setText(2,sLenght);
                it->setText(3,sSize);
                it->setText(4,sClips);
                it->setText(5,"0");
                it->setText(6,sRate);
                it->setText(7,sBitDepth);
                it->setText(8,sChannels);
                }
        s->recreate();
        PEXIT;
        }


void ProjectManager::update_projects_list()
        {
        listAlbumProjects->clear();

        QString projsDir(getenv("HOME"));
        projsDir = projsDir + "/protuxprojects/";

        QDir pDir( projsDir );

        const QFileInfoList* files = pDir.entryInfoList();
        if ( files )
                {
                QFileInfoListIterator it( *files );
                QFileInfo* fi;
                while( (fi=it.current()) != 0 )
                        {
                        ++it;
                        if ( (fi->fileName() != ".") && (fi->fileName() != "..") )
                                {
                                QListViewItem * item = new QListViewItem( listAlbumProjects, 0 );
                                item->setText( 0, fi->fileName() );
                                }
                        }
                }
        }


// delete file/dir pName after prepending $HOME/protuxprojects/ to it
//
// if it is a directory, calls itself recursively  on any file/dir in the directory
// before removing the directory
int ProjectManager::remove_recursively(QString pName)
        {
        QString name(getenv("HOME"));
        name = name + "/protuxprojects/" + pName;

        // check if we are removing the currentProject, and delete it before removing its files
        if (project_is_current(pName))
                {
                if (currentProject->get_current_song()->get_status() != Mixer::STOPPED)
                        {
                        PERROR("current song is not stopped\n");
                        PEXIT;
                        return -1;
                        }
                PMESG("removing current project\n");
                delete currentProject;
                currentProject = NULL;
                }

        QFileInfo fileInfo(name);

        if (!fileInfo.exists())
        return -1;
        if (!fileInfo.isWritable())
                {
                PERROR("failed to remove %s: you don't have write access to it\n", name.latin1());
                return -1;
                }

        if(fileInfo.isFile())
                {
                QFile file(name);
                if (!file.remove())
                        {
                        PERROR("failed to remove file %s\n", name.latin1());
                        return -1;
                        }
                return 0;
                }
        else if(fileInfo.isDir())
                {
                QDir dir(name);
                const QFileInfoList *files = dir.entryInfoList();
                if (files)
                        {
                        QFileInfoListIterator it( *files);
                        QFileInfo *fi;
                        while ((fi = it.current()) != 0)
                                {
                                QString filename(fi->fileName());
                                if ((filename != ".") && (filename != ".."))
                                        {
                                        remove_recursively(pName + "/" + filename);
                                        }
                                ++it;
                                }
                        }
                if (!dir.rmdir(name))
                        {
                        PERROR("failed to remove directory %s\n", name.latin1());
                        return -1;
                        }

                return 0;
                }

        return -1;
        }


int ProjectManager::copy_recursively(QString pNameFrom, QString pNameTo)
        {
        if (currentProject && (currentProject->get_current_song()->get_status() != Mixer::STOPPED))
                {
                PERROR("current song is not stopped\n");
                PEXIT;
                return -1;
                }

        QString nameFrom(getenv("HOME"));
        QString nameTo(nameFrom);
        nameFrom = nameFrom + "/protuxprojects/" + pNameFrom;
        nameTo = nameTo + "/protuxprojects/" + pNameTo;

        QFileInfo fileFromInfo(nameFrom);
        QFileInfo fileToInfo(nameTo);

        if (!fileFromInfo.exists())
                {
                PERROR("File or directory %s doesn't exist\n", pNameFrom.latin1());
                return -1;
                }
        if (fileToInfo.exists())
                {
                PERROR("File or directory %s already exists", pNameTo.latin1());
                return -1;
                }

        if(fileFromInfo.isFile())
                {
                QFile fileFrom(nameFrom);
                if (!fileFrom.open(IO_ReadOnly))
                        {
                        PERROR("failed to open file %s for reading\n", nameFrom.latin1());
                        return -1;
                        }

                QFile fileTo(nameTo);
                if (!fileTo.open(IO_WriteOnly))
                        {
                        fileFrom.close();
                        PERROR("failed to open file for writting%s\n", nameFrom.latin1());
                        return -1;
                        }

                // the real copy part should perhaps be implemented using QDataStream
                // but .handle() will still be needed to get the optimal block-size
                //
                //! \todo does not keep file mode yet
                int bufferSize = 4096;
                int fileDescFrom = fileFrom.handle();
                int fileDescTo = fileTo.handle();
                struct stat fileStat;
                if (fstat(fileDescFrom, &fileStat) == 0)
                        bufferSize = (int)fileStat.st_blksize;

                void *buffer = malloc(sizeof(char) * bufferSize);
                // QMemArray<char> buffer(bufferSize);

                for (;;)
                        {
                        int nRead = read(fileDescFrom, buffer, bufferSize);
                        if (nRead < 0)
                                {
                                fileFrom.close();
                                fileTo.close();
                                PERROR("Error while reading file %s\n", nameFrom.latin1());
                                return -1;
                                }
                        if (nRead == 0)
                                break;
                        if (write(fileDescTo, buffer, nRead) < 0)
                                {
                                fileFrom.close();
                                fileTo.close();
                                PERROR("Error while writing file %s\n", nameTo.latin1());
                                return -1;
                                }
                        }
                free(buffer);

                fileFrom.close();
                fileTo.close();

                return 0;
                }
        else if(fileFromInfo.isDir())
                {
                QDir dirFrom(nameFrom);
                QDir dirTo(nameTo);
                if (!dirTo.mkdir(nameTo))
                        {
                        PERROR("failed to create directory %s\n", nameTo.latin1());
                        return -1;
                        }

                const QFileInfoList *files = dirFrom.entryInfoList();
                if (files)
                        {
                        QFileInfoListIterator it( *files);
                        QFileInfo *fi;
                        while ((fi = it.current()) != 0)
                                {
                                QString filename(fi->fileName());
                                if ((filename != ".") && (filename != ".."))
                                        {
                                        copy_recursively(pNameFrom + "/" + filename, pNameTo + "/" + filename);
                                        }
                                ++it;
                                }
                        }
                return 0;
                }
        return -1;
        }



void ProjectManager::set_current_project(Project* pProject)
        {
        currentProject = pProject;
        if (currentProject)
                {
                btnMaster->setEnabled( true );
                btnSaveProject->setEnabled( true );
                fieldAlbumTitle->setText( currentProject->get_title() );
                QString sNOfSongs;
                sNOfSongs.setNum(currentProject->get_number_of_songs());
                fieldNOfSongs->setText(sNOfSongs);
                fieldEngineer->setText(currentProject->get_engineer());
                // TODO : load all project settings and put them into the widgets...
                // done... at least for the currently available settings ;-) /Martin
                }
        else
                {
                btnMaster->setEnabled( FALSE );
                btnSaveProject->setEnabled( FALSE );
                }
        }


int ProjectManager::create_new_project(QString projectName, int numSong)
        {
        PENTER;

        if (currentProject && (currentProject->get_current_song()->get_status() != Mixer::STOPPED))
                {
                PERROR("current song is not stopped\n");
                PEXIT;
                return -1;
                }

        if(projectExists(projectName))
                {
                PERROR("project %s already exists\n", projectName.latin1());
                PEXIT;
                return -1;
                }

        Project *newProject = new Project(projectName, parentInterface);
        if (!newProject)
                return -1;

        if (newProject->create(numSong) < 0)
                {
                delete newProject;
                PERROR("couldn't create new project %s", projectName.latin1());
                PEXIT;
                return -1;
                }

        if (currentProject)
                delete currentProject;

        set_current_project(newProject);
        parentInterface->set_current_project(newProject);
        currentProject->save();

        PEXIT;
        return 0;
        }


int ProjectManager::load_project(QString projectName)
        {
        PENTER;

        if (currentProject && (currentProject && (currentProject->get_current_song()->get_status() != Mixer::STOPPED)))
                {
                PERROR("current song is not stopped\n");
                PEXIT;
                return -1;
                }

        if(!projectExists(projectName))
                {
                PERROR("project %s doesn't exist\n", projectName.latin1());
                PEXIT;
                return -1;
                }

        Project *newProject = new Project(projectName, parentInterface);
        if (!newProject)
                return -1;

        if (newProject->load() < 0)
                {
                delete newProject;
                PERROR("couldn't load project %s", projectName.latin1());
                PEXIT;
                return -1;
                }

        if (currentProject)
                delete currentProject;

        set_current_project(newProject);
        parentInterface->set_current_project(newProject);

        PEXIT;
        return 0;
        }


bool ProjectManager::project_is_current(QString title)
        {
        QString path(getenv("HOME"));
        path = path + "/protuxprojects/" + title;

        if (currentProject && (currentProject->get_root_dir() == path))
                return true;
        return false;
        }




bool ProjectManager::projectExists(QString title)
        {
        QString path(getenv("HOME"));
        QFileInfo fileInfo(path + "/protuxprojects/" + title);
        if (fileInfo.exists())
                return true;
        return false;
        }


// ------------------------------- xxx_clicked()

void ProjectManager::tab_changed()
        {
        if (mainTabPane->currentPageIndex()==2)
                {
                update_audio_sources_list();
                }
        }


void ProjectManager::listAlbumProject_projectSelected()
        {
        QListViewItem *item = NULL;

        if ((item = listAlbumProjects->currentItem()) == 0)
                return;

        fieldSaveProjAs->setText(item->text(0));
        }


void ProjectManager::btnCreateProject_clicked()
        {
        // is current song stopped ?
        if (currentProject && (currentProject->get_current_song()->get_status() != Mixer::STOPPED))
                {
                QMessageBox::warning(this,
                                "Protux - Warning",
                                "This operation is not permitted while a song is playing,\nrecording, or some tracks are armed",
                                "Ok");
                return;
                }

        // do we have the name of the project to create ?
        QString title;
        title = fieldSaveProjAs->text();
        if (title.length() == 0)
                {
                QMessageBox::warning(this,
                                "Protux - Warning",
                                "You must supply a name for the project",
                                "Ok");
                return;
                }

        // ask if the current project should first be saved
        // todo: only ask if project has not been saved
        if (currentProject)
                switch (QMessageBox::information(this,
                                                "Protux - Question",
                                                "Should the current project first be saved ?",
                                                "Yes", "No", QString::null, 0, -1))
                        {
                        case -1:
                                return;
                                break;
                        case 0:
                                currentProject->save();
                                break;
                        default:
                                break;
                        }

        // first test if project exists already
        if (projectExists(title))
                {
                switch (QMessageBox::information(this,
                                                "Protux - Question",
                                                "The project \"" + title + "\" already exists, do you want to remove it and replace it with a new one ?",
                                                "Yes", "No", QString::null, 1, -1))
                        {
                        case 0:
                                remove_recursively(title);
                                break;
                        default:
                                return;
                                break;
                        }
                }

        if( create_new_project(title, 2) < 0)
                QMessageBox::warning(this,
                                "Protux - Error", "Couldn't create project \"" + title +"\"",
                                "Ok");
        update_projects_list();
        }


void ProjectManager::btnImportNow_clicked()
        {
        if (currentProject)
                {
                currentProject->import_audio(fileBrowser->filename->text(),pbarFileImport);
                }
        }


void ProjectManager::btnSaveProject_clicked()
        {
        // is current song stopped ?
        if (currentProject && (currentProject->get_current_song()->get_status() != Mixer::STOPPED))
                {
                QMessageBox::warning(this,
                                "Protux - Warning",
                                "This operation is not permitted while a song is playing,\nrecording, or some tracks are armed",
                                "Ok");
                return;
                }

        if (currentProject)
                {
                currentProject->set_title(fieldAlbumTitle->text());
                currentProject->set_engineer(fieldEngineer->text());
                currentProject->save();
                QMessageBox::warning( 0, "Project Saved", "Project \"" + currentProject->title + "\" saved.");
                }
        }


void ProjectManager::btnLoadProject_clicked()
        {
        // is current song stopped ?
        if (currentProject && (currentProject->get_current_song()->get_status() != Mixer::STOPPED))
                {
                QMessageBox::warning(this,
                                "Protux - Warning",
                                "This operation is not permitted while a song is playing,\nrecording, or some tracks are armed",
                                "Ok");
                return;
                }

        // do we have the name of the project to load ?
        QString title;
        title = fieldSaveProjAs->text();
        if (title.length() == 0)
                {
                QMessageBox::warning(this,
                                "Protux - Warning",
                                "You must supply a name for the project",
                                "Ok");
                return;
                }

        // ask if the current project should first be saved
        //! \todo only ask if project has not been saved
        if (currentProject)
                switch (QMessageBox::information(this,
                                                "Protux - Question",
                                                "Should the current project first be saved ?",
                                                "Yes", "No", QString::null, 0, -1))
                        {
                        case -1:
                                return;
                                break;
                        case 0:
                                currentProject->save();
                                break;
                        default:
                                break;
                        }

        // first test if project exists
        if (!projectExists(title))
                {
                QMessageBox::warning(this,
                                "Protux - Warning",
                                "The project \"" + title + "\" doesn't exist",
                                "Ok");
                update_projects_list();
                return;
                }

        load_project(title);
        }


void ProjectManager::btnCopyToProject_clicked()
        {
        // is current song stopped ?
        if (currentProject && (currentProject->get_current_song()->get_status() != Mixer::STOPPED))
                {
                QMessageBox::warning(this,
                                "Protux - Warning",
                                "This operation is not permitted while a song is playing,\nrecording, or some tracks are armed",
                                "Ok");
                return;
                }

        // do we have the name of the project to create ?
        if (!currentProject)
                return; // display a warning

        QString projectTo;
        projectTo = fieldSaveProjAs->text();
        if (projectTo.length() == 0)
                {
                QMessageBox::warning(this,
                                "Protux - Warning",
                                "You must supply a name for the project",
                                "Ok");
                return;
                }

        // the current project should first be saved
        //! \todo only ask if project has not been saved
        if (currentProject)
                switch (QMessageBox::information(this,
                                                "Protux - Question",
                                                "The current project has first to be saved before you can copy it !\nFuture versions will hopefully get rid of this constraint.",
                                                "Ok", "Cancel", QString::null, 0, -1))
                        {
                        case 0:
                                currentProject->save();
                                break;
                        default:
                                return;
                                break;
                        }

        // first test if project exists already
        if (projectExists(projectTo))
                {
                switch (QMessageBox::information(this,
                                                "Protux - Question",
                                                "The project \"" + projectTo + "\" already exists, do you want to remove it and replace it with the current one ?",
                                                "Yes", "No", QString::null, 1, -1))
                        {
                        case 0:
                                remove_recursively(projectTo);
                                break;
                        default:
                                return;
                                break;
                        }
                }

        QString projectFrom(currentProject->get_root_dir());
        // projectFrom = projectFrom.section("/", -1); // was not compatible with Qt 2.3, replaced with following 2 lines
        int index = projectFrom.findRev("/", -1);
        projectFrom = projectFrom.mid(index + 1);
        copy_recursively(projectFrom, projectTo);
        update_projects_list();
        load_project(projectTo);
        }


void ProjectManager::btnDeleteProject_clicked()
        {
        // do we have the name of the project to delete ?
        QString title;
        title = fieldSaveProjAs->text();
        if (title.length() == 0)
                {
                QMessageBox::warning(this,
                                "Protux - Warning",
                                "You must supply a name for the project",
                                "Ok");
                return;
                }

        // if deleting current project, is current song stopped ?
        if ((project_is_current(title)) && currentProject && (currentProject->get_current_song()->get_status() != Mixer::STOPPED))
                {
                QMessageBox::warning(this,
                                "Protux - Warning",
                                "This operation is not permitted while a song is playing,\nrecording, or some tracks are armed",
                                "Ok");
                return;
                }

        // first test if project exists
        if (!projectExists(title))
                {
                QMessageBox::warning(this,
                                "Protux - Error",
                                "The project \"" + title + "\" doesn't exist",
                                "Ok");
                update_projects_list();
                return;
                }

        switch (QMessageBox::information(this,
                                        "Protux - Question",
                                        "Are you sure that you want to remove the project \"" + title + "\" ? It's not possible to undo it !",
                                        "Yes", "No", QString::null, 1, -1))
                {
                case 0:
                        remove_recursively(title);
                        update_projects_list();
                        break;
                default:
                        return;
                        break;
                }
        return;
        }



void ProjectManager::btnSaveSong_clicked()
        {
        DevelGuardian::warn_not_implemented("SAVE SONG","Protux saves all songs under the current project automatically , for now");
        }

void ProjectManager::btnSaveSongAs_clicked()
        {
        DevelGuardian::warn_not_implemented("SAVE SONG AS","Sorry dude, no tips here...");
        }

void ProjectManager::btnSaveSongCopyAs_clicked()
        {
        DevelGuardian::warn_not_implemented("COPY SONG COPY AS","No tip for this lack.. sorry :-}");
        }


void ProjectManager::btnDeleteSource_clicked()
        {
        PENTER;
        //Remove current selected audio source in audioListView;
        QListViewItem* it = audioListView->currentItem();
        QString s = it->text(0);
        //      PDEBUG("TODO : Deleting audio source for file %s",s.ascii());
        Song* cs = parentInterface->get_current_project()->get_current_song();
        Audio* a = cs->get_asl()->get_audio_for_source(s);
        cs->remove_all_clips_for_audio(a);
        cs->get_asl()->remove(a);
        delete a;
        cs->recreate();
        PEXIT;
        }


void ProjectManager::btnRemoveAllClips_clicked()
        {
        DevelGuardian::warn_not_implemented("REMOVE ALL CLIPS","Don't hate us, but you will have to delete all them manually");
        }


void ProjectManager::btnExportSource_clicked()
        {
        DevelGuardian::warn_not_implemented("EXPORT SOURCE","Try to convert the audio source manually with genpraf, ogg123, mpg123, notlame, or some other console tool");
        }


void ProjectManager::btnSourcePlay_clicked()
        {
        DevelGuardian::warn_not_implemented("PLAY SOURCE","You can normally play it after loading it, but there no way to play it here in the Project Manager, for now...");
        }


void ProjectManager::btnFilePlay_clicked()
        {
        DevelGuardian::warn_not_implemented("FILE PLAY (INSIDE PROJECT MANAGER)","Use standard play tools to play it outside protux");
        }

void ProjectManager::btnAddFileAsSource_clicked()
        {
        DevelGuardian::warn_not_implemented("ADD AS AUDIO SOURCE","Import it normally then it will become an audio source");
        }

void ProjectManager::btnAddSourceCDA_clicked()
        {
        DevelGuardian::warn_not_implemented("Import CDA track as audio source","Use \"cdda2wav -D/dev/cdrom -tX\" in\na console to import it (Note: X is the number of the track)");
        }

void ProjectManager::btnImportNowCDA_clicked()
        {
        DevelGuardian::warn_not_implemented("Import CDA track as audio source","Use \"cdda2wav -D/dev/cdrom -tX\" in\na console to import it (Note: X is the number of the track)");
        }

void ProjectManager::btnCDAPlay_clicked()
        {
        DevelGuardian::warn_not_implemented("CD PLAYER","Use the KDE or the Gnome CD player");
        }

void ProjectManager::btnRender_clicked()
        {
        DevelGuardian::warn_not_implemented("FULL PROJECT RENDER","You can render each song a time by <<ENTER>> in them");
        }

void ProjectManager::btnMaster_clicked()
        {
        DevelGuardian::warn_not_implemented("MASTER PROJECT","This is feature for a very distant future release.");
        }

void ProjectManager::btnBurnCD_clicked()
        {
        DevelGuardian::warn_not_implemented("BURN CD ","We suggest xcdroast to burning CDs");
        }


void ProjectManager::audioListView_itemRenamed( QListViewItem *, int )
        {

        }

void ProjectManager::audioListView_spacePressed( QListViewItem * )
        {

        }


void ProjectManager::audioListView_returnPressed( QListViewItem * )
        {

        }


void ProjectManager::audioListView_onItem( QListViewItem * )
        {

        }


void ProjectManager::audioListView_collapsed( QListViewItem * )
        {

        }


void ProjectManager::audioListView_expanded( QListViewItem * )
        {

        }

// eof
