/**************************************************************************************

        PROTUX - THE FREE PROFESSIONAL AUDIO TOOLS FOR LINUX
        AUTHOR : See AUTHORS file for details

        This software is distributed under the terms of the GNU General Public License
        as specified in the COPYING file.

***************************************************************************************/

#include <mustux.h>
#include "Track.hh"
#include "ColorManager.hh"

// TODO : ALL lists stuff should be a single super class.. for example
// clips, regions, selections, and so on...

Track::Track(Song* pParentSong, int pID, QString pName, int pBaseY, int pHeight ) : MustuxDrawableRegion( pParentSong->get_tracks_area() )
        {
        PENTERCONS;
        parentSong = pParentSong;
        mixer = parentSong->get_mixer();
        name = pName;
        ID = pID;
        baseY = pBaseY;
        height = pHeight;
        clipList = (AudioClip*) 0;
        filterChain = new FilterChain(this);
        isActive = false;
        isMuted = false;
        isSolo = false;
        isArmed=false;
        busIn = 0;
        busOut = 0;
        pan=0;
        gain=0;
        lastRecordedClip = (AudioClip*) 0;
        whichChannels = MustuxAudioDeviceMapper::STEREO; // could be ONLY_LEFT_CHANNEL or ONLY_RIGHT_CHANNEL
        int b = 0;
        while (b<MustuxAudioDeviceMapper::get_total_buses_in())
                {
                if (mixer->valid_capture_bus(b))
                        {busIn=b; break;}
                b++;
                }
        b=0;
        while (b<MustuxAudioDeviceMapper::get_total_buses_out())
                {
                if (mixer->valid_playback_bus(b))
                        {busOut=b; break;}
                b++;
                }
        if (busIn<0)
                {
                PWARN("No Valid capture buses found for this Song settings");
                }
        if (busOut<0)
                {
                PWARN("No Valid playback buses found for this Song settings");
                }

        updateTimer = new QTimer(this);
        connect( updateTimer , SIGNAL(timeout()), this, SLOT(update_recording_clip()));

        PEXITCONS;
        }


Track::~Track()
        {
        PENTERDES;
        delete filterChain;
        PEXITDES;
        }


void Track::set_assoc_track_panel(TrackPanel* pAssocTrackPanel)
        {
        assocTrackPanel = pAssocTrackPanel;
        }


void Track::recreate()
        {
        PENTER3;
        clear();
        draw();
        update();
        assocTrackPanel->recreate();
        PEXIT3;
        }



void Track::update()
        {
        if (visible())
                {
                drawArea->update(0,real_baseY(),drawArea->width(),height);
                }
        }



void Track::clear()
        {
        PENTER3;
        if (visible())
                {
                drawArea = parentSong->get_tracks_area();
                QPainter* p = drawArea->painter;
                if (isActive)
                        p->fillRect(0, real_baseY(), drawArea->width() , height, QColor(160,140,140));
                else
                        p->fillRect(0, real_baseY(), drawArea->width() , height, ColorManager::get(ColorManager::COLOR_TRACK_BG));
                p->setPen(QColor(100,100,100));// This is intentionally fixed color
                p->drawLine(0, real_baseY() + height, drawArea->width(), real_baseY() + height);
                p->setPen(QColor(240,240,240));// This is intentionally fixed color.
                p->drawLine(0, real_baseY() + height -1 , drawArea->width(), real_baseY() + height -1);

/* FIXME               // Draw region markers
                int xrs = parentSong->block_to_xpos(parentSong->regionStart);
                int xre = parentSong->block_to_xpos(parentSong->regionEnd);
                int xs,xe=0;
                int w=drawArea->width();
                if ((xrs>=0) && (xrs<w))
                        xs=xrs;
                else if (xrs>=0)
                        xs=w;
                else
                        xs=0;

                if ((xre>=0) && (xre<w))
                        xe=xre;
                else if (xre>=0)
                        xe=w;
                else
                        xe=0;

                p->setPen(QColor(255,0,0));
                p->fillRect(xs,real_baseY(),xe-xs,3,QColor(255,40,255));
                drawArea->update(xs,real_baseY(),xe-xs,3);
  */
                }
        PEXIT3;
        }




int Track::draw()
        {
        PENTER3;
        if (visible())
                {
                AudioClip* c=clipList;
                while (c)
                        {
                        c->draw();
                        c=c->next;
                        }
                if (parentSong->editingMode==Song::EDITING_MODE_TRACK_CURVES)
                        filterChain->draw();
                PEXIT3;
                return 1;
                }
        PEXIT3;
        return 0; // 0 means no need of drawing
        }



bool Track::is_pointed()
        {
        drawArea = parentSong->tracksArea;
        int x = drawArea->mapFromGlobal(QCursor::pos()).x();
        int y = drawArea->mapFromGlobal(QCursor::pos()).y();
        bool b = ( (x >= 0)  &&
                   (x <= drawArea->width() ) &&
                   (y >= real_baseY() ) &&
                   (y <= real_baseY() + height )
                 );
        return b ;
        }



int Track::split_clip(long long splitPoint)
        {
        PENTER2;
        PMESG("Trying to find clip under %d",splitPoint);
        AudioClip* c = get_clip_under(splitPoint);
        if (c!=0)
                {
                long long leftLenght = splitPoint - (c->trackFirstBlock);
                long long clipLenght = c->sourceLastBlock - c->sourceFirstBlock;
                long long rightLenght = clipLenght - leftLenght;
                long long rightSourceFirstBlock = c->sourceFirstBlock + leftLenght;
                c->sourceLastBlock = c->sourceFirstBlock + leftLenght;
                c->trackLastBlock = c->trackFirstBlock + leftLenght;
                add_clip(c->get_audio_source(), splitPoint, rightSourceFirstBlock, rightLenght, c->isTake);
                }
        else
                {
                PMESG("I am upon no clip :-(");
                }
        recreate();
        parentSong->parentProject->get_parent_interface()->get_project_manager()->update();
        PEXIT2;
        return 0;
        }




AudioClip* Track::get_clip_under(long long blockPos)
        {
        PENTER4;
        AudioClip* theClip = (AudioClip*) 0;
        AudioClip* c=clipList;
        while (c)
                {
                long long fb = c->trackFirstBlock;
                long long lb = c->trackLastBlock;
                if ((fb<=blockPos) && (lb>=blockPos))
                        theClip = c;
                if (fb>blockPos) break;
                c=c->next;
                }
        PEXIT4;
        return theClip;
        }



AudioClip* Track::get_clip_after(long long blockPos)
        {
        PENTER3;
        AudioClip* c=clipList;
        while (c)
                {
                if (c->trackFirstBlock > blockPos)
                        return c;
                c=c->next;
                }
        PEXIT3;
        return (AudioClip*) 0;
        }


        
AudioClip* Track::get_clip_before(long long blockPos)
        {
        PENTER3;
        AudioClip* c  = clipList;
        AudioClip* c2 = (AudioClip*) 0;
        while (c2)
                {
                if (c->trackFirstBlock > blockPos)
                        return c2;
                c2=c;
                c=c->next;
                }
        PEXIT3;
        return (AudioClip*) 0;
        }




int Track::delete_clip(AudioClip* clip)
        {
        PENTER;
        if (!clip)
                {
                PERROR("Trying to delete invalid Clip");
                PEXIT;
                return -1;
                }
        if (!clip->prev) // if it is the first clip...
                {
                clipList=clip->next;
                if (clipList)
                        clipList->prev=0;
                delete clip;
                PEXIT;
                return 1;
                }
        if (clip->next==0) // if it is the last clip...
                {
                if (!clip->prev) // oops it is also the first ! (it is a single clip in the track)
                        {
                        delete clip;
                        clipList=0;
                        }
                else
                        {
                        clip->prev->next=0;
                        delete clip;
                        }
                PEXIT;
                return 1;
                }
        clip->prev->next = clip->next;
        clip->next->prev = clip->prev;
        delete clip;

        // JUST FOR DEBUG
        PMESG3("Entire clip list for track now is ");
        AudioClip* c=clipList;
        int i=0;
        while (c)
                {
                PMESG3("Clip # %d ",i);
                i++;
                c=c->next;
                }

        parentSong->update_last_block();
        PEXIT;
        return 1;
        }



AudioClip* Track::add_clip(Audio* audioSource, long long trackInsertBlock, bool isTake)
        {
        PENTER;
        PMESG("Creating clip starting in %d" ,trackInsertBlock);
        AudioClip* c = add_clip(audioSource, trackInsertBlock, 0, audioSource->file->totalBlocks,isTake);
        recreate();
        PEXIT;
        return c;
        }



AudioClip* Track::add_clip(Audio* audioSource, long long trackInsertPos,long long sourceFirstBlock, long long lenght, bool isTake)
        {
        PENTER;
        AudioClip *clip = new AudioClip(this,
                                        audioSource,
                                        sourceFirstBlock,
                                        lenght,
                                        trackInsertPos);
        clip->isTake = isTake;
        AudioClip* c;
        AudioClip* c2;

        if (!clipList)
                {
                long long clipLenght = clip->sourceLastBlock - clip->sourceFirstBlock;
                /*PMESG("Adding first clip :  [ TFB = " << clip->trackFirstBlock <<
                                        " , TLB = " << clip->trackLastBlock <<
                                        " , SFB = " << clip->sourceFirstBlock <<
                                        " , SLB = " << clip->sourceLastBlock <<
                                        " , LEN = " << clipLenght << " ]");
                */
                clipList = clip;
                clip->prev=0;
                clip->next=0;
                }
        else
                {
                c=clipList;
                c2=clipList;
                while (c &&  (c->trackFirstBlock < trackInsertPos))
                        {
                        c2=c;
                        c=c->next;
                        }
                if (!c)
                        {
                        PMESG("Adding at the end of list");
                        c2->next=clip;
                        clip->prev=c2;
                        clip->next=0;
                        parentSong->update_last_block();
                        }
                else
                        {
                        if (c==clipList)
                                {
                                PMESG("Adding at the begin of list");
                                clip->next=clipList;
                                clipList->prev=clip;
                                clip->prev=0;
                                clipList=clip;
                                }
                        else
                                {
                                PMESG("Adding in the middle of list");
                                clip->next=c2->next;
                                clip->prev=c2;
                                c2->next->prev=clip;
                                c2->next=clip;
                                }
                        }
                }
        PEXIT;
        return clip;
        }

void Track::activate()
        {
        if (!isActive)
                {
                isActive=true;
                int cfc  = filterChain->currentFilterController;
                if (cfc>=0)
                        filterChain->filterController[cfc]->start_blinking();
                }

        }


void Track::deactivate()
        {
        if (isActive)
                {
                int cfc  = filterChain->currentFilterController;
                if (cfc>=0)
                        filterChain->filterController[cfc]->deactivate();
                isActive=false;
                }
        }




void Track::set_blur(bool stat)
        {
        ColorManager::set_blur(ColorManager::COLOR_TRACK_BG , stat);
        AudioClip* c=clipList;
        while (c)
                {
                c->set_blur(stat);
                c=c->next;
                }
        }


int Track::get_baseX() // the implementation for track is simply return 0
        {
        return 0;
        }

int Track::get_baseY()
        {
        return baseY;
        }

int Track::get_width() // the implementation for tracks is simply return the drawArea widths
        {
        return drawArea->width();
        }


int Track::get_height()
        {
        return height;
        }


QString Track::get_name()
        {
        return name;
        }



int Track::arm()
        {
        PENTER;
        if ((mixer->get_status()==Mixer::STOPPED) ||
            (mixer->get_status()==Mixer::ARMED))
                {
                QString captureDir = parentSong->captureDir;
                QDate d = QDate::currentDate();
                QTime t = QTime::currentTime();
                QString s1;s1.setNum(d.year());
                QString s2;s2.setNum(d.month());
                QString s3;s3.setNum(d.day());
                QString s4;s4.setNum(t.hour());
                QString s5;s5.setNum(t.minute());
                QString s6;s6.setNum(t.second());
                QString tns = "take-"+s1+s2+s3+"-"+s4+s5+s6+".praf";
                recordingAudioFilename = tns;
                if (mixer->arm(busIn,whichChannels)<0)
                        {
                        PERROR("Cannot arm track");//improve me
                        PEXIT;
                        return -1;
                        }
                start_recording_clip();
                isArmed=true;
                recreate();
                }
        PEXIT;
        return 1;
        }




int Track::disarm()
        {
        PENTER;
        if ((mixer->get_status()==Mixer::STOPPED) || (mixer->get_status()==Mixer::ARMED)) // this avoid the disarm/arm during going state
                {
                isArmed=false;
                if (mixer->disarm(busIn)<0)
                        {
                        PERROR("Cannot disarm track");
                        PEXIT;
                        return -1;
                        }
                abort_recording_clip();
                recreate();
                }
        PEXIT;
        return 1;
        }


int Track::cleanup_buffers() // This is called everytime Mixer stops.
        {
        filterChain->cleanup();
        if (isArmed)
                {
                finish_recording_clip();
                QDate d = QDate::currentDate();
                QTime t = QTime::currentTime();
                QString s1;s1.setNum(d.year());
                QString s2;s2.setNum(d.month());
                QString s3;s3.setNum(d.day());
                QString s4;s4.setNum(t.hour());
                QString s5;s5.setNum(t.minute());
                QString s6;s6.setNum(t.second());
                QString tns = "take-"+s1+s2+s3+"-"+s4+s5+s6+".praf";
                recordingAudioFilename = tns;
                start_recording_clip(); // the new one :-), with a new name ;-)
                }
        }


Song* Track::get_parent_song()
        {
        return parentSong;
        }


int Track::get_bus_in()
        {
        return busIn;
        }

int Track::get_bus_out()
        {
        return busOut;
        }

int Track::get_which_channels()
        {
        return whichChannels;
        }

void Track::set_which_channels(int wc)
        {
        whichChannels = wc; // TODO : validate it..
        }

void Track::set_bus_in(int b)
        {
        bool wasArmed=isArmed;
        if (isArmed)
                disarm();
        busIn=b;
        if (wasArmed)
                arm();
        }

void Track::set_bus_out(int b)
        {
        busOut=b;
        }


bool Track::is_muted()
        {
        return isMuted;
        }

bool Track::is_solo()
        {
        return isSolo;
        }


bool Track::armed()
        {
        return isArmed;
        }

bool Track::is_active()
        {
        return isActive;
        }

void Track::toggle_active()
        {
        isActive=!isActive;
        }

QString Track::get_schema()
        {
        PENTER;
        QString sint;
        QString output="";
        AudioClip* cli;
        sint.setNum(ID);
        output = output + "<track id=\"" + sint + "\"";
        output = output + " name=\"" + name + "\"";
        QString sgain; sgain.setNum(gain,'g',3);
        QString span; span.setNum(pan,'g',3);
        output = output + " gain=" + sgain + " pan="+ span;
        output = output + " mute=" + (isMuted?"1":"0") + " solo=" + (isSolo?"1":"0");
        QString sBusIn; sBusIn.setNum(busIn);
        QString sBusOut; sBusOut.setNum(busOut);
        output = output + " busIn="+ sBusIn + " busOut=" + sBusOut;
        output = output +  ">\n";
        cli = clipList;
        QString tag;
        while (cli)
                {
                if (cli->isTake)
                        tag="take";
                else
                        tag="source";
                output = output +  "       <clip " + tag + "=\"" + cli->audioSource->get_pure_filename() + "\"";

                sint.setNum((double)cli->trackFirstBlock);
                output = output +  " TFB=" + sint;
                sint.setNum((double)cli->sourceFirstBlock);
                output = output +  " SFB=" + sint;
                sint.setNum((double)cli->get_lenght());
                output = output +  " LEN=" + sint + ">\n";
                cli=cli->next;
                }
        output = output +  filterChain->get_schema();
        output = output +  "</track>\n";
        PEXIT;
        return output;
        }


void Track::set_schema(QString schema)
        {
        PENTER;
        QTextStream ts( &schema, IO_ReadOnly );
        while (!ts.atEnd())
                {
                QString line = ts.readLine();

                int a=line.find("name=\"")+6;
                int b=line.find("\"",a);
                QString pname=line.mid(a,b-a);

                a=line.find("mute=")+5;
                b=line.find(" ",a+1);
                QString sMute=line.mid(a,b-a);
                bool mute=(sMute=="1");

                a=line.find("solo=")+5;
                b=line.find(" ",a+1);
                QString sSolo=line.mid(a,b-a);
                bool solo=(sSolo=="1");

                a=line.find("gain=")+5;
                b=line.find(" ",a+1);
                QString sGain=line.mid(a,b-a);
                float gain=sGain.toFloat();

                a=line.find("pan=")+4;
                b=line.find(" ",a+1);
                QString sPan=line.mid(a,b-a);
                float pan=sPan.toFloat();

                a=line.find("busIn=")+6;
                b=line.find(" ",a+1);
                QString sBusIn=line.mid(a,b-a);
                int busIn=sBusIn.toInt();

                a=line.find("busOut=")+7;
                b=line.find(" ",a+1);
                QString sBusOut=line.mid(a,b-a);
                int busOut=sBusOut.toInt();

                isMuted=mute;
                isSolo=solo;
                set_gain(gain);
                set_pan(pan);
                set_bus_in(busIn);
                set_bus_out(busOut);

                while ((line.find("</track")<0) && (!ts.atEnd()))
                        {
                        line = ts.readLine();
                        line = line.stripWhiteSpace();
                        if (line.find("<clip")>=0)
                                {
                                bool isTake;
                                int x1 = line.find("source=");
                                if (x1<0)
                                        {
                                        x1 = line.find("take=")+6;
                                        isTake=true;
                                        }
                                else
                                        {
                                        x1+=8;
                                        isTake=false;
                                        }
                                int x2 = line.find("\"", x1+1);
                                QString sSource=line.mid(x1,x2-x1);
                                int x3 = line.find("TFB=")+4;
                                int x4 = line.find(" ", x3+1);
                                QString sTFB=line.mid(x3,x4-x3);
                                int x5 = line.find("SFB=")+4;
                                int x6 = line.find(" ", x5+1);
                                QString sSFB=line.mid(x5,x6-x5);
                                int x7 = line.find("LEN=")+4;
                                int x8 = line.find(" ", x7+1);
                                QString sLEN=line.mid(x7,x8-x7);
                                long long tfb = (long long) sTFB.toDouble();
                                long long sfb = (long long) sSFB.toDouble();
                                long long len = (long long) sLEN.toDouble();

                                PMESG("Creating clip from [%s] TFB=%d SFB=%d LEN=%d",(const char*) sSource.latin1(),tfb,sfb,len );
                                Audio* audioSource = parentSong->audioSourcesList->get_audio_for_source(sSource);
                                if (!audioSource)
                                        {
                                        audioSource = new Audio(parentSong);
                                        int r = audioSource->build(parentSong->rootDir+(isTake?"/capture":"/audiosources"),sSource);
                                        if (r>0)
                                                {
                                                parentSong->audioSourcesList->add(audioSource);
                                                add_clip(audioSource, tfb, sfb, len, isTake);
                                                }
                                        }
                                else
                                        add_clip(audioSource, tfb, sfb, len, isTake);
                                }
                        else if (line.find("<filtercontroller")>=0)
                                {
                                int x1=line.find("type=")+6;
                                int x2=line.find("\"",x1);
                                QString fctype=line.mid(x1,x2-x1);
                                FilterController* fc = filterChain->add_filter_controller(fctype);
                                while ( fc && (line.find("</filtercontroller")<0) && (!ts.atEnd()))
                                        {
                                        line = ts.readLine();
                                        line = line.stripWhiteSpace();
                                        if (line.find("<curve")>=0)
                                                {
                                                int x1=line.find("type=")+6;
                                                int x2=line.find("\"",x1);
                                                QString cutype=line.mid(x1,x2-x1);
                                                PMESG("Adding Curve %s",(const char*)cutype.latin1());
                                                Curve* cu = fc->add_curve(cutype);
                                                if (!cu) // curve already exists !
                                                        cu = fc->get_curve(cutype); // so use the already existent curve

                                                while (cu && (line.find("</curve")<0) && (!ts.atEnd()))
                                                        {
                                                        line = ts.readLine();
                                                        line = line.stripWhiteSpace();
                                                        if (line.find("<node")>=0)
                                                                {
                                                                int x1=line.find("pos=")+4;
                                                                int x2=line.find(" ",x1);
                                                                QString spos=line.mid(x1,x2-x1);
                                                                int x3=line.find("value=")+6;
                                                                int x4=line.find(" ",x3);
                                                                QString sval=line.mid(x3,x4-x3);
                                                                PMESG("Adding node : [%s@%s]",(const char*)sval.latin1(),(const char*)spos.latin1());
                                                                long long pos= (long long) spos.toDouble();
                                                                float val= sval.toFloat();
                                                                CurveNode* no=cu->add_node(pos,val);
                                                                }
                                                        }
                                                }
                                        }
                                }
                        }
                }
        PEXIT;
        }


int Track::start_recording_clip()
        {
        PENTER;
        int recordingChannels = (whichChannels==MustuxAudioDeviceMapper::STEREO?2:1);
        recordingAudio = new Audio(parentSong);
        recordingAudio->create(parentSong->captureDir, recordingAudioFilename, recordingChannels, parentSong->get_rate(), parentSong->get_bit_depth(),"Untitled", "UndefGroup", "UndefSoundCard","Real Time Take");
        // IMPROVE LINE BELOW. File is already isolated within Audio
        recordingAudioFile = recordingAudio->file;
        recordingAudioFile->rewind_audio();
        recordingAudioStartPos = parentSong->get_working_block();
        recordingAudioPos = recordingAudioStartPos;
        updateTimer->start(50);
        PEXIT;
        return 1; // improve me
        }


int Track::abort_recording_clip()
        {
        PENTER;
        delete recordingAudio;
        updateTimer->stop();
        PEXIT;
        return 0;
        }



int Track::feed_recording_clip(char* buffer, int bufferSize)
        {
        PENTER4;
        recordingBufferSize = bufferSize;
        recordingBuffer = buffer;
        if ( recordingAudioFile->write_fragment( recordingBuffer, recordingBufferSize) < 0)
                {
                PERROR("Cannot write to output file.");
                PEXIT4;
                return -1;
                }
        recordingAudioPos += recordingBufferSize / recordingAudioFile->blockSize;
        PEXIT4;
        return 1;
        }


int Track::finish_recording_clip()
        {
        PENTER;
        updateTimer->stop();
        int r=accept_recorded_clip();
        PEXIT;
        return r;
        }


void Track::update_recording_clip()
        {
        PENTER2;
        if (mixer->get_status()==Mixer::GOING)
                {
                PMESG2("Updating recording Audio %p",recordingAudio);
                int recordingChannels = (whichChannels==MustuxAudioDeviceMapper::STEREO?2:1);
                int rx = parentSong->block_to_xpos(recordingAudioPos);
                int totalSamples = recordingBufferSize/parentSong->get_block_size();
                int numSamplesToDraw = parentSong->block_to_xpos(totalSamples);
                QPainter* fpainter = drawArea->fastPainter;

                if (( rx >= 0 ) && (rx <= drawArea->width()))
                        {
                        if ( rx + numSamplesToDraw > drawArea->width() )
                                numSamplesToDraw = drawArea->width() - rx;
                        int recordingBufferStep = parentSong->get_zoom_step()*parentSong->get_block_size();
                        int sho,maxsho;
                        int chhalf = (height-12) / recordingChannels;
                        int x = rx - 2; // this 2 pixels is to prevent that the recording buffer being drawed is blurred by playing cursors, that is being drawed in the same position during record
                        int sampleCounter=0;
                        if (numSamplesToDraw==0)
                                {
                                recordingBufferStep = recordingBufferSize;
                                numSamplesToDraw=1;
                                }
                        char* posBuffer = recordingBuffer;
                        while ( sampleCounter < numSamplesToDraw )
                                {
                                for (int k=0; k < recordingChannels; k++)
                                        {
                                        int yc = real_baseY() + chhalf*(k+1) + 4 ;
                                        fpainter->setPen(Qt::white);
                                        fpainter->drawLine(x,yc,x,yc-chhalf+1);
                                        char* p = posBuffer + k*sizeof(short);
                                        maxsho=0;
                                        while ( p < posBuffer + recordingBufferStep )
                                                {
                                                sho = (int)*(short*)p;
                                                if ( sho > maxsho ) maxsho=sho;
                                                p += sizeof(short)*recordingChannels;
                                                }
                                        if ( maxsho >= 32767 )
                                                {
                                                maxsho = 32767;
                                                fpainter->setPen(Qt::red);
                                                inputLevel[k]=maxsho;
                                                }
                                        else
                                                fpainter->setPen(QColor(150,40,40));
                                        float f = (float) maxsho / 32768;
                                        int y = (int) (f*chhalf);
                                        fpainter->drawLine(x,yc,x,yc-y);
                                        }
                                posBuffer+=recordingBufferStep;
                                x++;
                                sampleCounter++;
                                }
                        }
                }
        PEXIT2;
        }



int Track::accept_recorded_clip()
        {
        PENTER;
        recordingAudioFile->fix_audio_size();
        recordingAudio->rebuild_peaks();
        AudioClip* clip = parentSong->create_clip(recordingAudio, true, this, recordingAudioStartPos);
        if (!clip)
                {
                PERROR("After recording, could not create clip with recorded audio! Continuing anyway ...");
                PEXIT;
                return -1;
                }
        lastRecordedClip = clip;
        parentSong->set_work_at(lastRecordedClip->trackLastBlock);
        recreate();
        PEXIT;
        return 1;
        }


int Track::discard_recorded_clip() // should be void...
        {
        delete recordingAudio;
        return 1;
        }


float Track::get_gain()
        {
        return gain;
        }


float Track::get_pan()
        {
        return pan;
        }


void Track::set_gain(float pGain)
        {
        if ( pGain < -24.0 )
                gain=-24.0;
        else
        if ( pGain > 24.0 )
                gain=24.0;
        else
                gain=pGain;
        }


void Track::set_pan(float pPan)
        {
        if ( pPan < -1.0 )
                pan=-1.0;
        else
        if ( pPan > 1.0 )
                pan=1.0;
        else
                pan=pPan;
        }


void Track::vzoom_in(int newBaseY)
        {
        baseY = newBaseY;
        if (1.1*height<500.0)
                height = (int) (1.10*height);
        recreate();
        }


void Track::vzoom_out(int newBaseY)
        {
        baseY = newBaseY;
        if ( 0.9 * height > MINIMUM_HEIGHT )
                height = (int) (0.90 * height);
        recreate();
        }


int Track::set_baseY(int b)
        {
        baseY = b;
        }


int Track::set_height(int h)
        {
        height = h;
        }


int Track::get_total_clips()
        {
        int c=0;
        AudioClip* cli = clipList;
        while (cli) { cli=cli->next; c++; }
        return c;
        }


bool Track::visible()
        {
        // IMPROVE ME
        if ( baseY + parentSong->get_mta_baseY() + height < 0 )
                return false;
        return true;
        }


int Track::real_baseY() // return baseY with vertical offset applied
        {
        return baseY + parentSong->get_mta_baseY();
        }
// eof


